<?php
	// See doc/ for in-depth details

// ABOUT {{{
/** 
 * Transforms mail archives into XHTML pages and XML feeds.
 *
 * @package xhtmail
 * @author Patrice Levesque
 * @version 0.5
 * @copyright xhtmail comes with ABSOLUTELY NO WARRANTY and xhtmail may be copied only under the terms of the GNU General Public Licence, which may be found in the xhtmail distribution.
 * @license http://opensource.org/licenses/gpl-license.php GNU Public License
 */
// }}}
// CONFIGURATION {{{
// define('XHTMAIL_IMAGES_BASE_URI') {{{
/**
 * URI for the external objects (icons, stylesheets)
 * @name XHTMAIL_IMAGES_BASE_URI
 */
define('XHTMAIL_IMAGES_BASE_URI', 'http://ptaff.ca/xhtmailimg/');
// }}}
// define('XHTMAIL_PROCESS_XFACES') {{{
/**
 * Shall we process X-Faces (TRUE/FALSE)? if so we need to se paths for icontopbm, uncompface and convert
 * @name XHTMAIL_PROCESS_XFACES
 */
define('XHTMAIL_PROCESS_XFACES', TRUE);
// }}}
// define('XHTMAIL_ICONTOPBM') {{{
/**
 * Full path to icontopbm binary (needed if XHTMAIL_PROCESS_XFACES is set to TRUE)
 * @name XHTMAIL_ICONTOPBM
 */
define('XHTMAIL_ICONTOPBM', '/usr/bin/icontopbm');
// }}}
// define('XHTMAIL_UNCOMPFACE') {{{
/**
 * Full path to uncompface binary (needed if XHTMAIL_PROCESS_XFACES is set to TRUE)
 * @name XHTMAIL_UNCOMPFACE
 */
define('XHTMAIL_UNCOMPFACE', '/usr/bin/uncompface');
// }}}
// define('XHTMAIL_CONVERT') {{{
/**
 * Full path to convert binary (needed if XHTMAIL_PROCESS_XFACES is set to TRUE)
 * @name XHTMAIL_CONVERT
 */
define('XHTMAIL_CONVERT', '/usr/bin/convert');
// }}}
// define('XHTMAIL_XHTML_TO_TEXT') {{{
/**
 * Full path and options to a binary that converts from XHTML in stdin to text
 * @name XHTMAIL_XHTML_TO_TEXT
 */
define('XHTMAIL_XHTML_TO_TEXT', '/usr/bin/elinks -dump -dump-charset utf-8 -no-references -force-html -dump-width 60 -no-connect -no-numbering /dev/stdin');
// }}}
// define('XHTMAIL_DEBUG') {{{
/**
 * Message width for wordwrapping purposes
 * @name XHTMAIL_WORDWRAP_WIDTH
 */
define('XHTMAIL_WORDWRAP_WIDTH', 90);
/**
 * Shall we use debug mode (TRUE/FALSE)
 * @name XHTMAIL_DEBUG
 */
define('XHTMAIL_DEBUG', FALSE);
// }}}
// define('XHTMAIL_VERSION_NUMBER') {{{
/**
 * xhtmail version number
 * @name XHTMAIL_VERSION_NUMBER
 */
define('XHTMAIL_VERSION_NUMBER', "0.5");
// }}}
// define('XHTMAIL_VERSION_CODENAME') {{{
/**
 * xhtmail version codename
 * @name XHTMAIL_VERSION_CODENAME
 */
define('XHTMAIL_VERSION_CODENAME', "New faces");
// }}}
// define('XHTMAIL_COPYRIGHT') {{{
/**
 * xhtmail copyright string
 * @name XHTMAIL_COPYRIGHT
 */
define('XHTMAIL_COPYRIGHT', "xhtmail comes with ABSOLUTELY NO WARRANTY and xhtmail may be copied only under the terms of the GNU General Public Licence, which may be found in the xhtmail distribution.\n");
// }}}
// }}}
// GLOBALS (yuck!) {{{
/**
 * Function callback to the right translation function as set in parameters
 * Default is english (t_en) but can be overrided by commandline options
 * @global string $language_callback
 * @name $language_callback
 */
$language_callback = 't_en';
// }}}

	include('Mail/mimeDecode.php'); # make sure PEAR is in your path
	if (!function_exists('mailparse_msg_parse')) {
		dl('mailparse.so');
	};

	/* read commandline options */
	$options = parse_options();
	$language_callback = 't_' . $options['lang'];
	// create output directory if it does not exist
	mkdir_if_needed($options['output_directory']);
	// build internal index
	$index = index_build($options['files']);
	// build pages, attachments, so on...
	xhtml_build_thread_pages($index, $options);
	// build index (month, global) pages
	xhtml_build_index_pages($index, $options);
	exit();

// FUNCTIONS {{{
	
// function mkdir_if_needed($dir) {{{
/**
 * Creates a directory if it does not exist
 * @param string $dir The directory to create
 * @return boolean TRUE if the directory exists or is successfully created; FALSE on error
 */
function mkdir_if_needed($dir) {
	if (!file_exists($dir)) {
		@mkdir($dir, 0755) or die("Tried to create the `" . $dir . "` directory but it failed.  Check permissions?");
	};
	return TRUE;
}; 
// }}}
// function xhtml_build_thread_pages(&$index, $options) {{{
/**
 * Builds thread pages
 * @param array &$index Index array
 * @param array $options Global options
 * @see function index_build
 * @see function parse_options
 */
function xhtml_build_thread_pages(&$index, $options) { 
	// index is passed by ref cause body decoding will change it.
	$indexdata = array();
	$feeds = array();
	$rss_path = '';
	$atom_path = '';
	$rss_button = '';
	$atom_button = '';
	$first_filename = '';
	$first_title = '';
	$prev_filename = '';
	$prev_title = '';
	if ($options['feed'] > 0) {
		$rss_path = $options['uri_base_feeds'] . '/feeds/index.rss';
		$atom_path = $options['uri_base_feeds'] . '/feeds/atom.xml';
		$rss_button = "<div class=\"rss\"><a href=\"{$options['uri_base_feeds']}/feeds/index.rss\">RSS</a></div>"; 
		$atom_button = "<div class=\"atom\"><a href=\"{$options['uri_base_feeds']}/feeds/atom.xml\">Atom</a></div>"; 
	};
	$index_count = count($index);
	for ($i = 0; $i < $index_count; $i++) {
		if (XHTMAIL_DEBUG) { echo "Building thread page for {$index[$i]['subject']}\n"; };
		// check for month directory and create if needed
		$output_filename_month_dir = date("Y-m", $index[$i]['unixdate']);
		mkdir_if_needed($options['output_directory'] . '/' . $output_filename_month_dir);
		// create file directory if needed
		$output_filename_dir = date("Y-m", $index[$i]['unixdate']);
		$thread_directory = $options['output_directory'] . '/' . $output_filename_month_dir . '/' . $index[$i]['unixdate'] . '-' . encode_utf8_filename_to_ascii($index[$i]['subject']);
		mkdir_if_needed($thread_directory);
		$output_filename = $thread_directory . '/index.' . $options['extension'];
		// add to index; ship previous and next in case...
		if ($i != 0) {
			$first_filename = '../' . date("Y-m", $index[0]['unixdate']) . '/' . $index[0]['unixdate'] . '-' . encode_utf8_filename_to_ascii($index[0]['subject']) . '/';
			$first_title = $index[0]['subject'];
		};
		if ($i > 0) { 
			$prev = $index[$i - 1]; 
			$prev_filename = '../' . date("Y-m", $index[$i - 1]['unixdate']) . '/' . $index[$i - 1]['unixdate'] . '-' . encode_utf8_filename_to_ascii($index[$i - 1]['subject']) . '/';
			$prev_title = $index[$i - 1]['subject'];
		} 
		else { 
			$prev = NULL; 
		};
		if ($i < ($index_count - 1)) { 
			$next = $index[$i + 1]; 
			$next_filename = '../' . date("Y-m", $index[$i + 1]['unixdate']) . '/' . $index[$i + 1]['unixdate'] . '-' . encode_utf8_filename_to_ascii($index[$i + 1]['subject']) . '/';
			$next_title = $index[$i + 1]['subject'];
		} 
		else { 
			$next = NULL; 
		};
		if ($i != ($index_count - 1)) {
			$last_filename = '../' . date("Y-m", $index[$index_count - 1]['unixdate']) . '/' . $index[$index_count - 1]['unixdate'] . '-' . encode_utf8_filename_to_ascii($index[$index_count - 1]['subject']) . '/';
			$last_title = $index[$index_count - 1]['subject'];
		};
		list($msg, $tree, $attributes) = xhtml_get_msg_and_tree_from_index_entry(1, $index[$i], $thread_directory, $options, $prev, $next);
		if ($options['feed'] > 0) {
			$feeds[] = array("title" => escape_xhtml_and_mail($index[$i]['subject']), "author" => escape_xhtml_and_mail(extract_author_name_from_email_header(decode_header_to_utf8($index[$i]['author']))), "pubdate" => $attributes['creation_date'], "moddate" => $attributes['update_date'], "x-url" => $attributes['x-url'], "link" => '/' . $output_filename_month_dir . '/' . $index[$i]['unixdate'] . '-' . encode_utf8_filename_to_ascii($index[$i]['subject']), "description" => $index[$i]['description'], "contributors" => $attributes['contributor']);
		};
		if ($options['after'] != '') {
			($after = implode('', file($options['after']))) || die ("Can't read `{$options['after']}` you specified.  Aborting.");
		};
		touch($output_filename) || die ("Can't create file.  Check your output directory parameter.  Aborting.");
		if ($options['before'] != '') {
			($before = implode('', file($options['before']))) || die ("Can't read `{$options['before']}` you specified.  Aborting.");
		};
		if ($options['contents-after'] != '') {
			($contents_after = implode('', file($options['contents-after']))) || die ("Can't read `{$options['contents-after']}` you specified.  Aborting.");
		};
		// metadata handling
		$attributes['creation_date'] = date("Y-m-d", $attributes['creation_date']);
		$attributes['update_date'] = date("Y-m-d", $attributes['update_date']);
		$attributes['contributor'] = filter_contributors($attributes['contributor']);
		$fout = fopen($output_filename, 'w');
		fwrite($fout, parse_template($before, array("MAILING_LIST_NAME" => $options['name_of_list'], "TITLE" => escape_xhtml_and_mail($index[$i]['subject']), "RSS_PATH" => $rss_path, "ATOM_PATH" => $atom_path, "INDEX_URI" => "../../", "INDEX_TITLE" => t("Index"), "PARENT_URI" => "../../" . date("Y-m", $index[$i]['unixdate']) . '/', "PARENT_TITLE" => $options['name_of_list'] . ' - ' . date("Y-m", $index[$i]['unixdate']), "FIRST_URI" => '../' . $first_filename, "FIRST_TITLE" => escape_xhtml_and_mail($first_title), "PREV_URI" => '../' . $prev_filename, "PREV_TITLE" => escape_xhtml_and_mail($prev_title), "NEXT_URI" => '../' . $next_filename, "NEXT_TITLE" => escape_xhtml_and_mail($next_title), "LAST_URI" => '../' . $last_filename, "LAST_TITLE" => escape_xhtml_and_mail($last_title), "DATE_OF_CREATION" => $attributes['creation_date'], "DATE_OF_UPDATE" => $attributes['update_date'], "CREATOR_NAME" => $attributes['creator'], "CONTRIBUTOR_NAMES" => flatten_contributor_names($attributes['contributor']))));
		fwrite($fout, "<div id=\"threadtree\"><div id=\"inthreadtree\">\n");
		fwrite($fout, $tree);
		fwrite($fout, $rss_button);
		fwrite($fout, $atom_button);
		fwrite($fout, "</div></div>\n");
		fwrite($fout, "<div id=\"bodies\"><div id=\"inbodies\">\n");
		fwrite($fout, $msg);
		fwrite($fout, "</div>\n");
		fwrite($fout, $contents_after);
		fwrite($fout, "</div>\n");
		fwrite($fout, $after);
		fclose($fout);
	};
	// write global feeds
	write_rss_to_file($feeds, $options);
	write_atom_to_file($feeds, $options);
}; 
// }}}
// function flatten_contributor_names($arr) {{{
/**
 * Takes a contributor list and returns a comma-delimited list of them
 * @param array $arr Array of contributors
 * @return string A comma-delimited list of contributors
 */
function flatten_contributor_names($arr) {
	$out = '';
	if (count($arr) == 0) { return ''; };
	foreach ($arr as $a) {
		$out .= "," . $a['name'];
	};
	return substr($out, 1);

}; 
// }}}
// function write_rss_to_file($feeds, $options) {{{
/**
 * Writes a RSS2.0 file
 * @param array $feeds List of feed infos
 * @param array $options Global options
 * @see function parse_options
 */
function write_rss_to_file($feeds, $options) { 
	$contents = "<" . "?xml version=\"1.0\" encoding=\"utf-8\"?" . ">\n";
	# Seems this doctype is invalid and I could not find any else
	#$contents .= "<!DOCTYPE rss PUBLIC \"-//Netscape Communications//DTD RSS 2.0//EN\" \"http://smallstreams.com/publish/formats/rss-2.0.dtd\">\n";
	$contents .= "<rss version=\"2.0\" xmlns:dc=\"http://purl.org/dc/elements/1.1/\">\n";
	$contents .= "\t<channel>\n";
	$contents .= "\t\t<generator>xhtmail " . XHTMAIL_VERSION_NUMBER . ' [' . XHTMAIL_VERSION_CODENAME . "]</generator>\n";
	$contents .= "\t\t<pubDate>" . date("r") . "</pubDate>\n";
	$contents .= "\t\t<lastBuildDate>" . date("r") . "</lastBuildDate>\n";
	$contents .= "\t\t<language>" . $options['lang'] . "</language>\n";
	$contents .= "\t\t<title>{$options['name_of_list']}</title>\n";
	$contents .= "\t\t<link>{$options['uri_base_web']}/</link>\n";
	if ($options['title_of_list'] != '') {
		$contents .= "\t\t<description>{$options['title_of_list']}</description>\n";
	};
	if ($options['picture_uri'] != '') {
		$contents .= "\t\t<image>\n";
		$contents .= "\t\t\t<url>{$options['picture_uri']}</url>\n";
		$contents .= "\t\t\t<title>{$options['name_of_list']}</title>\n";
		$contents .= "\t\t\t<link>{$options['uri_base_web']}/</link>\n";
		$contents .= "\t\t</image>\n";
	};
	for ($i = (count($feeds) - 1); (($i >= 0) && ($i > (count($feeds) - 1 - $options['feed']))); $i--) {
		$r = $feeds[$i];
		$contents .= "\t\t<item>\n";
		$contents .= "\t\t\t<title>" . myxmlentities(utf8_decode($r['title'])) . "</title>\n";
		$contents .= "\t\t\t<link>{$options['uri_base_web']}" . myurlencode($r['link']) . "/</link>\n";
		$contents .= "\t\t\t<guid>{$options['uri_base_web']}" . myurlencode($r['link']) . "/</guid>\n";
		$contents .= "\t\t\t<description>" . myxmlentities(utf8_decode($r['description'])) . "</description>\n";
		$contents .= "\t\t\t<pubDate>" . date("r", $r['pubdate']) . "</pubDate>\n";
		$contents .= "\t\t\t<dc:creator>" . $r['author'] . "</dc:creator>\n";
		$contents .= "\t\t</item>\n";
	};
	$contents .= "\t</channel>\n";
	$contents .= "</rss>\n";
	mkdir_if_needed($options['output_directory'] . '/feeds');
	$fout = fopen($options['output_directory'] . '/feeds/index.rss', 'w');
	fwrite($fout, $contents);
	fclose($fout);
}; 
// }}}
// function write_atom_to_file($feeds, $options) {{{
/**
 * Writes an Atom 1.0 file
 * @param array $feeds List of feed infos
 * @param array $options Global options
 * @see function parse_options
 */
function write_atom_to_file($feeds, $options) { 
	$contents = "<" . "?xml version=\"1.0\" encoding=\"utf-8\"?" . ">\n";
	$contents .= "<feed xmlns=\"http://www.w3.org/2005/Atom\" xml:lang=\"{$options['lang']}\">\n";
	$contents .= "\t<!--\n";
	$contents .= "\t\tSome elements used here are NOT part of the Atom 1.0 (Version 10) specification but\n";
	$contents .= "\t\tare used for compatibility with UAs that do not comply yet:\n";
	$contents .= "\t\t\tfeed::modified\t(is now replaced by feed::updated)\n";
	$contents .= "\t\t\tfeed::issued\t(is now replaced by feed::published)\n";
	$contents .= "\t\t\tfeed::tagline\t(is now replaced by feed::subtitle)\n";
	$contents .= "\t-->\n";
	$contents .= "\t<title	 type=\"text\">{$options['name_of_list']}</title>\n";
	$contents .= "\t<generator uri=\"http://ptaff.ca/xhtmail/\" version=\"" . XHTMAIL_VERSION_NUMBER . "\">xhtmail " . XHTMAIL_VERSION_NUMBER . ' [' . XHTMAIL_VERSION_CODENAME . "]</generator>\n";
	$contents .= "\t<link	  rel=\"alternate\" type=\"text/html\" href=\"{$options['uri_base_web']}/\" title=\"{$options['name_of_list']}\" />\n";
	$contents .= "\t<link	  rel=\"alternate\" type=\"application/xhtml+xml\" href=\"{$options['uri_base_web']}/\" title=\"{$options['name_of_list']}\" />\n";
	$contents .= "\t<link	  rel=\"self\" type=\"application/atom+xml\" href=\"{$options['uri_base_feeds']}/feeds/atom.xml\" title=\"{$options['name_of_list']}\" />\n";
	if ($options['title_of_list'] != '') {
		$contents .= "\t<subtitle  type=\"text\">{$options['title_of_list']}</subtitle>\n";
		$contents .= "\t<tagline   type=\"text\">{$options['title_of_list']}</tagline>\n";
	};
	$contents .= "\t<id>" . $options['uri_base_web'] . "/</id>\n";
	$contents .= "\t<updated>" . get_iso8601_date(time()) . "</updated>\n"; # MUST
	if ($options['picture_uri'] != '') {
		$contents .= "\t<logo>" . $options['picture_uri'] . "</logo>\n";
	};
	if ($options['icon_uri'] != '') {
		$contents .= "\t<icon>" . $options['icon_uri'] . "</icon>\n";
	};
	for ($i = (count($feeds) - 1); (($i >= 0) && ($i > (count($feeds) - 1 - $options['feed']))); $i--) {
		$r = $feeds[$i];
		$contents .= "\t<entry>\n";
		$contents .= "\t\t<id>{$options['uri_base_web']}" . myurlencode($r['link']) . "/</id>\n";
		$contents .= "\t\t<issued>" . get_iso8601_date($r['pubdate']) . "</issued>\n";
		$contents .= "\t\t<published>" . get_iso8601_date($r['pubdate']) . "</published>\n";
		$contents .= "\t\t<updated>" . get_iso8601_date($r['moddate']) . "</updated>\n";
		$contents .= "\t\t<modified>" . get_iso8601_date($r['moddate']) . "</modified>\n";
		$contents .= "\t\t<author>\n";
		$contents .= "\t\t\t<name>{$r['author']}</name>\n";
		if ($r['x-url'] != '') {
			$contents .= "\t\t\t<uri>{$r['x-url']}</uri>\n";
		};
		$contents .= "\t\t</author>\n";
		if (count($r['contributors']) > 0) {
			foreach ($r['contributors'] as $c) {
				$contents .= "\t\t<contributor>\n";
				$contents .= "\t\t\t<name>{$c['name']}</name>\n";
				if ($c['x-url'] != '') {
					$contents .= "\t\t\t<uri>" . myurlencode($c['x-url']) . "</uri>\n";
				};
				$contents .= "\t\t</contributor>\n";
			};
		};
		$contents .= "\t\t<title   type=\"text\">" . myxmlentities(utf8_decode($r['title'])) . "</title>\n";
		$contents .= "\t\t<link	rel=\"alternate\" href=\"{$options['uri_base_web']}" . myurlencode($r['link']) . "/\" type=\"text/html\" />\n";
		$contents .= "\t\t<link	rel=\"alternate\" href=\"{$options['uri_base_web']}" . myurlencode($r['link']) . "/\" type=\"application/xhtml+xml\" />\n";
		// Atom is made to support embedded namespaces but it does not work yet
		$contents .= "\t\t<summary type=\"html\">" . myxmlentities(utf8_decode($r['description'])) . "</summary>\n";
		$contents .= "\t</entry>\n";
	};
	$contents .= "</feed>\n";
	mkdir_if_needed($options['output_directory'] . '/feeds');
	$fout = fopen($options['output_directory'] . '/feeds/atom.xml', 'w');
	fwrite($fout, $contents);
	fclose($fout);
}; 
// }}}
// function xhtml_build_index_pages($index, $options) {{{
/**
 * Writes monthly and global index pages
 * @param array $index The internal index
 * @param array $options Global options
 * @see function index_build
 * @see function parse_options
 */
function xhtml_build_index_pages($index, $options) { 
	$output_index = $options['output_directory'] . '/index.' . $options['extension'];
	touch($output_index) || die ("Can't create index file.  Check your output directory parameter.  Aborting.");
	// build link list
	if ($options['after'] != '') {
		($after = implode('', file($options['after']))) || die ("Can't read `{$options['after']}` you specified.  Aborting.");
	};
	touch($output_index) || die ("Can't create file.  Check your output directory parameter.  Aborting.");
	if ($options['before'] != '') {
		($before = implode('', file($options['before']))) || die ("Can't read `{$options['before']}` you specified.  Aborting.");
	};
	if ($options['index_after'] != '') {
		($index_after = implode('', file($options['index_after']))) || die ("Can't read `{$options['index_after']}` you specified.  Aborting.");
	};
	$foutdex = fopen($output_index, 'w');
	$rssline_for_months = '';
	$rssline_for_index = '';
	$atomline_for_months = '';
	$atomline_for_index = '';
	$rss_path = '';
	$atom_path = '';
	if ($options['feed'] > 0) {
		$rssline_for_months = "<div class=\"rss\"><a href=\"{$options['uri_base_feeds']}/feeds/index.rss\">RSS</a></div>\n";
		$rssline_for_index = "<div class=\"rss\"><a href=\"{$options['uri_base_feeds']}/feeds/index.rss\">RSS</a></div>\n";
		$rss_path = $options['uri_base_feeds'] . '/feeds/index.rss';
		$atomline_for_months = "<div class=\"atom\"><a href=\"{$options['uri_base_feeds']}/feeds/atom.xml\">Atom</a></div>\n";
		$atomline_for_index = "<div class=\"atom\"><a href=\"{$options['uri_base_feeds']}/feeds/atom.xml\">Atom</a></div>\n";
		$atom_path = $options['uri_base_feeds'] . '/feeds/atom.xml';
	};
	fwrite($foutdex, parse_template($before, array("MAILING_LIST_NAME" => $options['name_of_list'], "RSS_PATH" => $rss_path, "ATOM_PATH" => $atom_path, "TITLE" => $options['title_of_list'])));
	fwrite($foutdex, "<h1 id=\"sectiontitle\">" . $options['title_of_list'] . "</h1>\n");
	fwrite($foutdex, "<ol id=\"mainindex\">\n");
	$indexlines = array();
	$monthlines = array();
	// index is ordered.  create different pages
	$curmonth = '';
	$msgcount = 0;
	$msgblurb = '';
	// build month index for previous and next links
	$month_index = array();
	foreach ($index as $i) {
		$month = date("Y-m", $i['unixdate']);
		if (!in_array($month, $month_index)) {
			$month_index[] = $month;
		};
	};
	// build index files and month files
	foreach ($index as $i) {
		$msg_in_thread = count_msg_in_thread($i);
		if (date('Y-m', $i['unixdate']) != $curmonth) {
			// is this NOT the first ever file? if so, close the last file
			// and add to entry.  DUPLICATE CODE, after the loop.
			if ($curmonth != '') {
				$indexlines[] = "<li><a href=\"" . $curmonth . '/' . "\">" . $curmonth . "</a> <span class=\"threadsinmonth\">" . sprintf(tn('[%d thread]', '[%d threads]', $msgcount), $msgcount) . "</span><ul class=\"blurbs\">$msgblurb</ul></li>\n";
				$msgcount = 0;
				$msgblurb = '';
				fwrite($fmonth, implode('', array_reverse($monthlines)));
				$monthlines = array();
				fwrite($fmonth, "</ol>\n");
				fwrite($fmonth, $rssline_for_months);
				fwrite($fmonth, $atomline_for_months);
				fwrite($fmonth, xhtml_prev_next_month(array("PARENT_URI" => "../", "PARENT_TITLE" => t("Index"), "PREV_URI" => $prev_filename, "PREV_TITLE" => escape_xhtml_and_mail($prev_title), "NEXT_URI" => $next_filename, "NEXT_TITLE" => escape_xhtml_and_mail($next_title)), 1));
				fwrite($fmonth, $index_after);
				fwrite($fmonth, $after);
				fclose($fmonth);
			};
			$curmonth = date('Y-m', $i['unixdate']);
			mkdir_if_needed($options['output_directory']);
			$fmonth = fopen($options['output_directory'] . '/' . $curmonth . '/index.' . $options['extension'], 'w');
			// search for previous and next...
			$first_filename = $first_title = $prev_filename = $prev_title = '';
			$last_filename = $last_title = $next_filename = $next_title = '';
			$ptr = array_search($curmonth, $month_index);
			if ($ptr > 0) { // we are not the first month
				$first_filename = '../' . $month_index[0] . '/';
				$first_title = $options['name_of_list'] . ' - ' . $month_index[0];
				$prev_filename = '../' . $month_index[$ptr - 1] . '/';
				$prev_title = $options['name_of_list'] . ' - ' . $month_index[$ptr - 1];
			};
			if ($ptr < (count($month_index) - 1)) { // we are not the last month
				$next_filename = '../' . $month_index[$ptr + 1] . '/';
				$next_title = $options['name_of_list'] . ' - ' . $month_index[$ptr + 1];
				$last_filename = '../' . $month_index[count($month_index) - 1] . '/';
				$last_title = $options['name_of_list'] . ' - ' . $month_index[count($month_index) - 1];
			};
			fwrite($fmonth, parse_template($before, array("MAILING_LIST_NAME" => $options['name_of_list'], "TITLE" => escape_xhtml_and_mail($curmonth), "RSS_PATH" => $rss_path, "ATOM_PATH" => $atom_path, "PARENT_URI" => "../", "PARENT_TITLE" => t("Index"), "FIRST_URI" => $first_filename, "FIRST_TITLE" => escape_xhtml_and_mail($first_title), "PREV_URI" => $prev_filename, "PREV_TITLE" => escape_xhtml_and_mail($prev_title), "NEXT_URI" => $next_filename, "NEXT_TITLE" => escape_xhtml_and_mail($next_title), "LAST_URI" => $last_filename, "LAST_TITLE" => escape_xhtml_and_mail($last_title))));
			fwrite($fmonth, "<h1 id=\"sectiontitle\">" . $options['title_of_list'] . ' [' . $curmonth . "]</h1>\n");
			fwrite($fmonth, xhtml_prev_next_month(array("PARENT_URI" => "../", "PARENT_TITLE" => t("Index"), "PREV_URI" => $prev_filename, "PREV_TITLE" => escape_xhtml_and_mail($prev_title), "NEXT_URI" => $next_filename, "NEXT_TITLE" => escape_xhtml_and_mail($next_title)), 0));
			fwrite($fmonth, "<ol id=\"mainindex\">\n");
		};

		$link_filename = $i['unixdate'] . '-' . encode_utf8_filename_to_ascii($i['subject']) . '/';
		$monthlines[] = "<li class=\"hatom\"><div class=\"hentry\"><span class=\"date published\" title=\"" . date('c', $i['unixdate']) . "\">" . date("Y-m-d", $i['unixdate']) . "</span> <a rel=\"bookmark\" class=\"threadtitle entry-title\" href=\"$link_filename\">" . escape_xhtml_and_mail($i['subject']) . "</a> <span class=\"msginthread\">" . sprintf(tn('[%d message in thread]', '[%d messages in thread]', $msg_in_thread), $msg_in_thread) . "</span> <span class=\"indexauthor vcard\"><span class=\"fn\">" . escape_xhtml_and_mail(extract_author_name_from_email_header(decode_header_to_utf8($i['author']))) . "</span></span>" . (($i['short_description'] != '') ? "<p class=\"entry-summary\">{$i['short_description']}</p>" : "") . "</div></li>";
		$msgcount++;
		if ($msgblurb == '') { 
			$msgblurb .= "<li class=\"first\"><a href=\"$curmonth/$link_filename\">" . escape_xhtml_and_mail($i['subject']) . "</a></li>";
		}
		else {
			$msgblurb .= "<li><a href=\"$curmonth/$link_filename\">" . escape_xhtml_and_mail($i['subject']) . "</a></li>";
		};
	};
	// close everything
	$indexlines[] = "<li><a href=\"" . date('Y-m', $i['unixdate']) . '/' . "\">" . date('Y-m', $i['unixdate']) . "</a> <span class=\"threadsinmonth\">" . sprintf(tn('[%d thread]', '[%d threads]', $msgcount), $msgcount) . "</span><ul class=\"blurbs\">$msgblurb</ul></li>\n";
	fwrite($fmonth, implode('', array_reverse($monthlines)));
	fwrite($fmonth, "</ol>\n");
	fwrite($fmonth, $rssline_for_months);
	fwrite($fmonth, $atomline_for_months);
	fwrite($fmonth, xhtml_prev_next_month(array("PARENT_URI" => "../", "PARENT_TITLE" => t("Index"), "PREV_URI" => $prev_filename, "PREV_TITLE" => escape_xhtml_and_mail($prev_title), "NEXT_URI" => $next_filename, "NEXT_TITLE" => escape_xhtml_and_mail($next_title)), 1));
	fwrite($fmonth, $index_after);
	fwrite($fmonth, $after);
	fclose($fmonth);
	// all processes.  close month file and index file.
	fwrite($foutdex, implode('', array_reverse($indexlines)));
	fwrite($foutdex, "</ol>\n");
	fwrite($foutdex, $rssline_for_index);
	fwrite($foutdex, $atomline_for_index);
	fwrite($foutdex, $index_after);
	fwrite($foutdex, $after);
	fclose($foutdex); 

}; 
// }}}
// function xhtml_prev_next_month($links, $order) {{{
/**
 * Builds XHTML list code for previous and next links in month views
 * The input hash contains this:
 * <ul>
 * <li><var>['NEXT_TITLE']	  </var> Title of the next link</li>
 * <li><var>['NEXT_URI']		</var> URI of the next link</li>
 * <li><var>['PARENT_TITLE']	</var> Title of the parent link</li>
 * <li><var>['PARENT_URI']	  </var> URI of the parent link</li>
 * <li><var>['PREV_TITLE']	  </var> Title of the previous link</li>
 * <li><var>['PREV_URI']		</var> URI of the previous link</li>
 * </ul>
 * @param array $links A hash containing the links info
 * @param boolean $order If false, parent link goes before prev/next; if true, parent link goes after prev/next
 * @return string The XHTML code
 */
function xhtml_prev_next_month($links, $order) {
	$out = "<ul class=\"navmonth\">\n";
	$index = "<li class=\"up\"><a href=\"{$links['PARENT_URI']}\"><img src=\"" . XHTMAIL_IMAGES_BASE_URI . "up.png\" width=\"8\" height=\"8\" alt=\"" . t('[Parent]') . "\" /> {$links['PARENT_TITLE']}</a></li>\n";
	if ($order == 0) { $out .= $index; };
	if ($links['PREV_URI'] != '') {
		$out .= "<li class=\"prev\"><a href=\"{$links['PREV_URI']}\"><img src=\"" . XHTMAIL_IMAGES_BASE_URI . "prev.png\" width=\"8\" height=\"8\" alt=\"" . t('[Prev]') . "\" /> {$links['PREV_TITLE']}</a></li>\n";
	};
	if ($links['NEXT_URI'] != '') {
		$out .= "<li class=\"next\"><a href=\"{$links['NEXT_URI']}\">{$links['NEXT_TITLE']} <img src=\"" . XHTMAIL_IMAGES_BASE_URI . "next.png\" width=\"8\" height=\"8\" alt=\"" . t('[Next]') . "\" /></a></li>\n";
	};
	if ($order == 1) { $out .= $index; };
	$out .= "</ul>\n";
	return $out;
}; 
// }}}
// function xhtml_get_msg_and_tree_from_index_entry($level, &$index_entry, $thread_directory, $options, $prev = NULL, $next = NULL) {{{
/**
 * Recursively unroll index entries and formats/fetches the related contents, trees, and attributes from the mail messages
 * Return array format:
 * <ul>
 * <li><var>[0]</var> Contents</li>
 * <li><var>[1]</var> Tree</li>
 * <li><var>[2]</var> Attributes</li>
 * </ul>
 * @param integer $level The depth of the message in the index entry; starts at 1
 * @param array &$index_entry The index entry to unroll; message ['description'] will be added to this hash if feeds are enabled, ['shortdesc'] will be nonetheless
 * @param string $thread_directory Directory where the thread will be stored; used for attachments
 * @param array $options Global options
 * @param array $prev The index entry of the previous message, if available
 * @param array $next The index entry of the next message, if available
 * @return array Contents, tree and attributes in the format indicated above
 * @see function index_build
 * @see function parse_options
 */
function xhtml_get_msg_and_tree_from_index_entry($level, &$index_entry, $thread_directory, $options, $prev = NULL, $next = NULL) {
	$tree = '';
	// index_entry is passed by ref to allow rss description passing.
	if (XHTMAIL_DEBUG) { echo "\tMsg building from {$index_entry['filename']}\n"; };
	$mime = mailparse_msg_parse_file($index_entry['filename']);
	$parts = mailparse_msg_get_structure($mime);
	$metadata = mailparse_msg_get_part_data($mime);
	$attach = array();
	$foundtextsection = FALSE;
	if (XHTMAIL_DEBUG) { echo "\tScanning parts\n"; };
	foreach ($parts as $p) {
		// find text section
		$section = mailparse_msg_get_part($mime, $p);
		$thismetadata = mailparse_msg_get_part_data($section);
		if (stristr($thismetadata['headers']['content-type'], 'text/plain')) {
			// if we already have a text/plain section, skip it!
			if (!$foundtextsection) {
				$foundtextsection = TRUE;
				ob_start();
				mailparse_msg_extract_part_file($section, $index_entry['filename']);
				$body = ob_get_contents();
				ob_end_clean();
				$body = mail_encoding_to_utf8($body, $thismetadata['headers']['content-type']);
			};
		}
		else {
			$attach[] = $section;
		};
	};
	if (!$foundtextsection) {
		// no text section, try to convert the first HTML version... grrrrr
		foreach ($parts as $p) {
			$section = mailparse_msg_get_part($mime, $p);
			$thismetadata = mailparse_msg_get_part_data($section);
			if ((stristr($thismetadata['headers']['content-type'], 'text/html')) || (stristr($thismetadata['headers']['content-type'], 'application/xhtml+xml'))) {
				// if we already have a text/plain section, skip it!
				if (!$foundtextsection) {
					$foundtextsection = TRUE;
					ob_start();
					mailparse_msg_extract_part_file($section, $index_entry['filename']);
					$htmlbody = ob_get_contents();
					ob_end_clean();
					$body = format_xhtml_from_mail($htmlbody);
				};
			};
		};
	};
	$body = mail_wordwrap($body, XHTMAIL_WORDWRAP_WIDTH);
	if (XHTMAIL_DEBUG) { echo "\tFinished scanning parts\n"; };
	if (XHTMAIL_DEBUG) { echo "\tDid it have a text section? " . ($foundtextsection ? 'Yes' : 'No') . "\n"; };
	$contents = '';
	// Init array.
	$attributes = array();
	$attributes['contributor'] = array();
	if ($level == 1) { // prev, next, creator, creation date
		$attributes['creator'] = escape_xhtml_and_mail(extract_author_name_from_email_header(decode_header_to_utf8($index_entry['author'])));
		$attributes['creation_date'] = $index_entry['unixdate'];
		$contents .= xhtml_add_prev_and_next($prev, $index_entry, $next, 0, $options);
		$tree .= "<div class=\"listname\">{$options['title_of_list']}</div>\n";
		$tree .= "<h1>" . escape_xhtml_and_mail($index_entry['subject']) . "</h1>\n";
		$tree .= "<ul class=\"tree\">\n";
	}
	else {
		$tmpcontributor = array();
		$tmpcontributor['name'] = escape_xhtml_and_mail(extract_author_name_from_email_header(decode_header_to_utf8($index_entry['author'])));
		$tmpcontributor['x-url'] = (isset($metadata['headers']['x-url']) ? $metadata['headers']['x-url'] : '');
		$attributes['contributor'][] = $tmpcontributor;
	};
	if (XHTMAIL_DEBUG) { echo "\tCode point A\n"; };
	$attributes['update_date'] = $index_entry['unixdate'];
	$contents .= "<div class=\"level\">\n";
	$contents .= xhtml_format_msg($level, $index_entry, $body, $metadata, $attach, $thread_directory, $options);
	$tree .= "<li><a class=\"singlelink\" href=\"#a" . md5($index_entry['id']) . "\">" . escape_xhtml_and_mail($index_entry['subject']) . "</a>\n";
	if (XHTMAIL_DEBUG) { echo "\tCode point B\n"; };
	if ((isset($index_entry['child'])) && (count($index_entry['child']) > 0)) {
		$tree .= "<ul>";
		$subtree = '';
		foreach ($index_entry['child'] as $c) {
			list($msg, $tmptree, $tmpattributes) = xhtml_get_msg_and_tree_from_index_entry($level + 1, $c, $thread_directory, $options);
			$subtree .= $tmptree;
			$contents .= $msg;
			// attributes, eeeeeh.  add contributor if not already there, update
			// last update date if above everything.
			if ($tmpattributes['update_date'] > $attributes['update_date']) {
				$attributes['update_date'] = $tmpattributes['update_date'];
			};
			$attributes['contributor'] = array_merge($attributes['contributor'], $tmpattributes['contributor']);
		};
		$tree .= $subtree;
		$tree .= "</ul>\n";
	};
	$tree .= "</li>\n";
	if ($level == 1) { // prev, next
		$tree .= "</ul>\n";
		$tree .= xhtml_add_prev_and_next_for_tree($prev, $index_entry, $next, $options);
	};
	$contents .= "</div>\n";
	if ($level == 1) { // prev, next
		$contents .= xhtml_add_prev_and_next($prev, $index_entry, $next, 1, $options);
	};
	$attributes['x-url'] = (isset($metadata['headers']['x-url']) ? $metadata['headers']['x-url'] : '');
	if (XHTMAIL_DEBUG) { echo "\tWill return from xhtml_get_msg_and_tree_from_index_entry\n"; };
	return array($contents, $tree, $attributes);
}; 
// }}}
// function xhtml_add_prev_and_next($prev, $current, $next, $order, $options) {{{
/**
 * Build XHTML list code for previous and next links in main thread views
 * @param array $prev Index entry to the previous index entry
 * @param array $current Index entry to the current index entry
 * @param array $next Index entry to the next index entry
 * @param boolean $order If false, parent link goes before prev/next; if true, parent link goes after prev/next
 * @param array $options Global options
 * @return string XHTML representation of the said list
 * @see function index_build
 * @see function parse_options
 */
function xhtml_add_prev_and_next($prev, $current, $next, $order, $options) {
	$contents = '';
	$prev_filename = '../../' . date("Y-m", $prev['unixdate']) . '/' . $prev['unixdate'] . '-' . encode_utf8_filename_to_ascii($prev['subject']) . '/';
	$next_filename = '../../' . date("Y-m", $next['unixdate']) . '/' . $next['unixdate'] . '-' . encode_utf8_filename_to_ascii($next['subject']) . '/';
	$indexparent = "<li class=\"up\"><a href=\"../../" . date("Y-m", $current['unixdate']) . '/' . "\"><img src=\"" . XHTMAIL_IMAGES_BASE_URI . "up.png\" width=\"8\" height=\"8\" alt=\"" . t('[Parent]') . "\" /> " . date("Y-m", $current['unixdate']) . "</a></li>\n";
	$indextop = "<li class=\"top\"><a href=\"../../" . "\"><img src=\"" . XHTMAIL_IMAGES_BASE_URI . "top.png\" width=\"8\" height=\"8\" alt=\"" . t('[Index]') . "\" /> " . t('Index') . "</a></li>\n";
	if ($order == 1) {
		$contents .= "<hr />";
	};
	$contents .= "<ul class=\"nav\">\n";
	if ($order == 0) { 
		$contents .= $indextop; 
		$contents .= $indexparent; 
	};
	if ($prev['subject'] != '') { 
		$contents .= "<li class=\"prev\"><a href=\"$prev_filename\"><img src=\"" . XHTMAIL_IMAGES_BASE_URI . "prev.png\" width=\"8\" height=\"8\" alt=\"" . t('[Prev]') . "\" /> " . escape_xhtml_and_mail($prev['subject']) . "</a></li>\n";
	};
	if ($next['subject'] != '') {
		$contents .= "<li class=\"next\"><a href=\"$next_filename\">" . escape_xhtml_and_mail($next['subject']) . " <img src=\"" . XHTMAIL_IMAGES_BASE_URI . "next.png\" width=\"8\" height=\"8\" alt=\"" . t('[Next]') . "\" /></a></li>\n";
	};
	if ($order == 1) { 
		$contents .= $indexparent; 
		$contents .= $indextop; 
	};
	$contents .= "</ul>\n";
	$contents .= "<div class=\"clearboth\">&#160;</div>\n";
	return $contents;
}; 
// }}}
// function xhtml_add_prev_and_next_for_tree($prev, $current, $next, $options) {{{
/**
 * Build XHTML list code for previous and next links in main tree views
 * @param array $prev Index entry to the previous index entry
 * @param array $current Index entry to the current index entry
 * @param array $next Index entry to the next index entry
 * @param array $options Global options
 * @return string XHTML representation of the said list
 * @see function index_build
 * @see function parse_options
 */
function xhtml_add_prev_and_next_for_tree($prev, $current, $next, $options) {
	$prev_filename = '../../' . date("Y-m", $prev['unixdate']) . '/' . $prev['unixdate'] . '-' . encode_utf8_filename_to_ascii($prev['subject']) . '/';
	$next_filename = '../../' . date("Y-m", $next['unixdate']) . '/' . $next['unixdate'] . '-' . encode_utf8_filename_to_ascii($next['subject']) . '/';
	$contents = "<ul class=\"nav\">\n";
	if ($prev['subject'] != '') { 
		$contents .= "<li class=\"prev\"><a href=\"$prev_filename\"><img src=\"" . XHTMAIL_IMAGES_BASE_URI . "prev.png\" width=\"8\" height=\"8\" alt=\"" . t('[Prev]') . "\" /> " . escape_xhtml_and_mail($prev['subject']) . "</a></li>\n";
	};
	if ($next['subject'] != '') {
		$contents .= "<li class=\"next\"><a href=\"$next_filename\"><img src=\"" . XHTMAIL_IMAGES_BASE_URI . "next.png\" width=\"8\" height=\"8\" alt=\"" . t('[Next]') . "\" /> " . escape_xhtml_and_mail($next['subject']) . "</a></li>\n";
	};
	$contents .= "<li class=\"up\"><a href=\"../../" . date("Y-m", $current['unixdate']) . '/' . "\"><img src=\"" . XHTMAIL_IMAGES_BASE_URI . "up.png\" width=\"8\" height=\"8\" alt=\"" . t('[Parent]') . "\" /> " . date("Y-m", $current['unixdate']) . "</a></li>\n";
	$contents .= "<li class=\"top\"><a href=\"../../" . "\"><img src=\"" . XHTMAIL_IMAGES_BASE_URI . "top.png\" width=\"8\" height=\"8\" alt=\"" . t('[Index]') . "\" /> " . t('Index') . "</a></li>\n";
	$contents .= "</ul>\n";
	$contents .= "<div class=\"clearboth\">&#160;</div>\n";
	return $contents;
}; 
// }}}
// function xhtml_format_msg($level, &$index_entry, $body, $metadata, $attach, $thread_directory, $options) {{{
/**
 * Builds the XHTML version of a message, including headers, icons, permlinks
 * @param integer $level Depth of the message, starting at 1
 * @param array &$index_entry Index entry of the message to build
 * @param string $body Body of the message
 * @param array $metadata Hash of the message metadata
 * @param array $attach Attachments
 * @param string $thread_directory Directory where the thread will be saved; used for attachments
 * @param array $options Global options
 * @return string XHTML representation of the message
 * @see function index_build
 * @see function parse_options
 */
function xhtml_format_msg($level, &$index_entry, $body, $metadata, $attach, $thread_directory, $options) {
	if (XHTMAIL_DEBUG) { echo "\tEntering xhtml_format_msg\n"; };
	// index_entry is passed by ref cause we'll add stuff in it.
	$contents = "<div class=\"hatom\" id=\"a" . md5($index_entry['id']) . "\">\n";
	$contents .= "<div class=\"hentry\">";
	$contents .= "<div class=\"msgheader\">";
	# MAIL TITLE
	if ($level < 7) {
		$contents .= "<h$level class=\"entry-title\">" . escape_xhtml_and_mail($index_entry['subject']) . "</h$level>\n";
	}
	else {
		$contents .= "<div class=\"h$level entry-title\">" . escape_xhtml_and_mail($index_entry['subject']) . "</div>\n";
	};
	if (XHTMAIL_DEBUG) { echo "\tFinished escaping subject\n"; };
	# MESSAGE HEADER BUTTONS
	$contents .= "<ul class=\"msgheadattributes\">\n";
	# country
	list($country_name, $tld) = extract_tld_from_email_header($metadata['headers']['from']);
	$contents .= "<li><a href=\"" . get_country_uri_from_tld($tld) . "\" title=\"$country_name\"><img class=\"flag\" src=\"" . XHTMAIL_IMAGES_BASE_URI . "flags/$tld.png\" width=\"14\" height=\"14\" alt=\"$country_name\" title=\"$country_name\" /></a></li>\n";
	# os of the user
	list($os_name, $os_code) = extract_os_from_email_header($metadata['headers']);
	if ($os_name != '') {
		$contents .= "<li><a href=\"" . get_os_uri($os_code) . "\" title=\"$os_name\"><img class=\"os\" src=\"" . XHTMAIL_IMAGES_BASE_URI . "os/$os_code.png\" width=\"14\" height=\"14\" alt=\"$os_name\" title=\"$os_name\" /></a></li>\n";
	};
	# mail user agent
	list($mua_name, $mua_code) = extract_mua_from_email_header($metadata['headers']);
	if ($mua_name != '') {
		$contents .= "<li><a href=\"" . get_mua_uri($mua_code) . "\" title=\"$mua_name\"><img class=\"mua\" src=\"" . XHTMAIL_IMAGES_BASE_URI . "mua/$mua_code.png\" width=\"14\" height=\"14\" alt=\"$mua_name\" title=\"$mua_name\" /></a></li>\n";
	};
	# foaf
	if ((isset($metadata['headers']['x-foaf-uri'])) && ($metadata['headers']['x-foaf-uri'] != '')) {
		$contents .= "<li><a href=\"http://xml.mfd-consult.dk/foaf/explorer/?foaf=" . myurlencode($metadata['headers']['x-foaf-uri']) . "\"><img class=\"foaf\" src=\"" . XHTMAIL_IMAGES_BASE_URI . "foaf.png\" width=\"14\" height=\"14\" alt=\"FOAF\" title=\"FOAF\" /></a></li>\n";
	};
	# pgp
	if ((isset($metadata['headers']['x-pgp-key'])) && ($metadata['headers']['x-pgp-key'] != '')) {
		$contents .= "<li><a href=\"{$metadata['headers']['x-pgp-key']}\"><img class=\"gpg\" src=\"" . XHTMAIL_IMAGES_BASE_URI . "key.png\" width=\"14\" height=\"14\" alt=\"PGP\" title=\"PGP\" /></a></li>\n";
	};
	$contents .= "</ul>\n";
	# END OF HEADER BUTTONS
	$contents .= "</div>\n";
	if (XHTMAIL_DEBUG) { echo "\tFinished header buttons\n"; };
	$contents .= "<div class=\"metadata\">\n";
	$contents .= "<div class=\"vcard\">\n";
	$contents .= xhtml_format_avatar($metadata, $options);
	$contents .= "<div class=\"author\">";
	$contents .= "<span class=\"fn\">";
	$from = escape_xhtml_and_mail(decode_header_to_utf8($metadata['headers']['from']));
	if (ereg(' &#60;', $from)) {
		$contents .= str_replace(" &#60;", "</span> &#60;", $from);
	}
	elseif (ereg(' &#60;', $from)) {
		$contents .= str_replace("&#60;", "</span> &#60;", $from);
	}
	else {
		$contents .= $from . "</span>";
	}
	$contents .= "</div>\n";
	if ((isset($metadata['headers']['x-url'])) && ($metadata['headers']['x-url'] != '')) {
		$contents .= "<div class=\"xurl\"><a class=\"url\" href=\"{$metadata['headers']['x-url']}\">{$metadata['headers']['x-url']}</a></div>\n";
	};
	$contents .= "</div>\n";
	$contents .= "<div class=\"date published\" title=\"" . date("c", $index_entry['unixdate']). "\">" . date("Y-m-d H:i:s T", $index_entry['unixdate']) . "</div>\n";
	$contents .= "<div class=\"clearboth\">&#160;</div>\n";
	$contents .= "</div>\n";
	if (XHTMAIL_DEBUG) { echo "\tFinished metadata\n"; };
	$body = htmlwrap(filter_body_to_xhtml($body), 80, "\n", "", "pre", TRUE);
	if (XHTMAIL_DEBUG) { echo "\tFinished body wrapping\n"; };
	if ($options['feed'] > 0) {
		$index_entry['description'] = get_description_from_body($body, 10);
	};
	$index_entry['short_description'] = get_description_from_body($body, 4);
	if (XHTMAIL_DEBUG) { echo "\tFinished body processing\n"; };
	$body = str_replace("\n", "<br />\n", $body);
	$contents .= "<div class=\"msgbody entry-content\">" . $body . "</div>\n";
	$contents .= "<div class=\"permlink\"><a rel=\"bookmark\" href=\"#a" . md5($index_entry['id']) . "\">" . t('Permalink') . "</a></div>\n";
	// attachments
	if (XHTMAIL_DEBUG) { echo "\tCode point C\n"; };
	$contents .= xhtml_unwrap_and_link_to_attachments($attach, $index_entry, $thread_directory, $options);
	if (XHTMAIL_DEBUG) { echo "\tCode point D\n"; };
	$contents .= "<div class=\"clearboth\">&#160;</div>\n";
	$contents .= "</div>\n";
	$contents .= "</div>\n";
	return $contents;
}; 
// }}}
// function xhtml_format_avatar($metadata, $options) {{{
function xhtml_format_avatar($metadata, $options) {
	$contents = '';
	if (isset($metadata['headers']['x-face']) && ($metadata['headers']['x-face'] != '') && XHTMAIL_PROCESS_XFACES) {
		$fallback = extract_xface_from_email_header($metadata['headers']['x-face'], $options);
	}
	elseif ($options['gravatar_fallback'] != '') {
		$fallback = $options['gravatar_fallback'];
	}
	else {
		$fallback = '';
	};
	$avatar_url = 'http://www.gravatar.com/avatar/' . md5(extract_email_from_from_header($metadata['headers']['from'])) . '?s=48';
	if ($fallback != '') {
		$avatar_url .= '&#38;d=' . urlencode($fallback);
	};
	$contents .= "<img class=\"xface photo\" src=\"$avatar_url\" width=\"48\" height=\"48\" alt=\"\" />\n";
	return $contents;
}; 
// }}}
// function xhtml_unwrap_and_link_to_attachments($attach, $index_entry, $thread_directory, $options) {{{
/**
 * Saves attachments to files and create a XHTML list of them
 * @param array $attach Attachments
 * @param array $index_entry Index entry of the message to build
 * @param string $thread_directory Directory where attachments will be saved
 * @param array $options Global options
 * @return string XHTML list of links to the attachments
 * @see function index_build
 * @see function parse_options
 */
function xhtml_unwrap_and_link_to_attachments($attach, $index_entry, $thread_directory, $options) {
	$contents = '';
	$atleastone = FALSE;
	foreach ($attach as $s) {
		$a = mailparse_msg_get_part_data($s);
		$filename = '';
		if ((isset($a['content-name'])) && ($a['content-name'] != '')) {
			$filename = encode_utf8_filename_to_ascii(decode_header_to_utf8($a['content-name']));
			$textname = decode_header_to_utf8($a['content-name']);
		}
		elseif ((isset($a['disposition-filename'])) && ($a['disposition-filename'] != '')) {
			$filename = encode_utf8_filename_to_ascii(decode_header_to_utf8($a['disposition-filename']));
			$textname = decode_header_to_utf8($a['disposition-filename']);
		}
		else {
			switch ($a['content-type']) {
				case 'text/html':
					// we skip.  too much trouble
					break;
				case 'multipart/alternative':
				case 'multipart/signed':
				case 'multipart/mixed':
				case 'multipart/related':
					// this is the whole mail.  Couldn't care less
					break;
				case 'application/pgp-keys':
					$filename = t('key') . '.pgp';
					$textname = t('PGP public key');
					break;
				case 'application/pgp-signature':
					$filename = t('signature') . '.pgp';
					$textname = t('PGP signature');
					break;
				case 'text/plain':
					$filename = t('untitled') . '.txt';
					$textname = t('Untitled text attachment');
					break;
		case 'message/rfc822':
			// was found once in a GMail reply.  Seems to be an inclusion
			// of the previous message; skip
			break;
				default:
					die("Don't know how to handle content-type {$a['content-type']}, in file '" . $index_entry['filename'] . "'.  Skipping");
			};
		};
		if ($filename != '') { // we can create
			if ($atleastone == FALSE) {
				$contents .= "<dl><dt>" . t('Attachments') . "</dt><dd>";
				$contents .= "<ul class=\"attach\">\n";
				$atleastone = TRUE;
			};
			$md5 = md5($index_entry['id']);
			$output_filename = $index_entry['unixdate'] . '-' . encode_utf8_filename_to_ascii($index_entry['subject']) . '-' . md5($index_entry['id']) . '-' . $filename;
			// make sure directory exists before creating subdirectory
			$fout = fopen($thread_directory . '/' . $output_filename, 'w');
			ob_start();
			mailparse_msg_extract_part_file($s, $index_entry['filename']);
			$partdata = ob_get_contents();
			ob_end_clean();
			fwrite($fout, $partdata);
			fclose($fout);
			if ($a['content-type'] != '') {
				$class = str_replace(".", "", str_replace("-", "", str_replace("/", "", $a['content-type'])));
			}
			else {
				$class = 'unknown';
			};
			$contents .= "<li class=\"$class\"><a href=\"$output_filename\">$textname</a></li>\n";
		};

	};
	if ($atleastone == TRUE) {
		$contents .= "</ul></dd></dl>\n";
	};
	return $contents;
}; 
// }}}
// function escape_xhtml_and_mail($contents) {{{
/**
 * Escapes '&', '<', '>' and '"' chars to entities and obfuscate email addresses
 * @param string $contents The contents to escape
 * @param boolean $arobas_too Encode arobas too
 * @return string The escaped string
 */
function escape_xhtml_and_mail($contents, $arobas_too = TRUE) {
	$contents = str_replace("&", "&#38;", $contents);
	$contents = str_replace(">", "&#62;", $contents);
	$contents = str_replace("<", "&#60;", $contents);
	$contents = str_replace('"', "&#34;", $contents);
	$contents = str_replace('$', "&#36;", $contents);
	if ($arobas_too) {
	$contents = filter_email_address($contents);
	}
	else {
		$contents = str_replace('@', "&#64;", $contents);

	};
	return $contents;
}; 
// }}}
// function filter_body_to_xhtml($contents) {{{
/**
 * Main function to markup and clean mail contents
 * Calls subfunctions that do the hard work
 * @param string $contents The mail body contents
 * @return string The converted contents
 */
function filter_body_to_xhtml($contents) {
	if (XHTMAIL_DEBUG) { echo "\tEntering filter_body_to_xhtml\n"; };
	$lines = explode("\n", $contents);
	// skip beginning empty lines
	if (XHTMAIL_DEBUG) { echo "\tCalling filter_body_to_xhtml_empty_lines\n"; };
	$lines = filter_body_to_xhtml_empty_lines($lines);
	// make nice URI
	if (XHTMAIL_DEBUG) { echo "\tCalling filter_body_to_xhtml_links\n"; };
	$lines = filter_body_to_xhtml_links($lines);
	// here, lines are < > and & encoded
	// quoting
	if (XHTMAIL_DEBUG) { echo "\tCalling filter_body_to_xhtml_quotes\n"; };
	$lines = filter_body_to_xhtml_quotes($lines);
	// signature
	if (XHTMAIL_DEBUG) { echo "\tCalling filter_body_to_xhtml_signature\n"; };
	$lines = filter_body_to_xhtml_signature($lines);
	// remove the jeopardy crap
	if (XHTMAIL_DEBUG) { echo "\tCalling filter_body_to_xhtml_jeopardy\n"; };
	$lines = filter_body_to_xhtml_jeopardy($lines);
	$output = implode("\n", $lines);
	// replace all arobases with an image 
	if (XHTMAIL_DEBUG) { echo "\tCalling filter_email_address\n"; };
	$output = filter_email_address($output);
	return $output;

}; 
// }}}
// function filter_body_to_xhtml_jeopardy($lines) {{{
/**
 * Removes useless whole messages quoted at the end
 * @param array $lines Lines of the message
 * @return array The cleaned up message
 */
function filter_body_to_xhtml_jeopardy($lines) {
	$lines2 = array();
	foreach ($lines as $l) {
		if ((trim($l) != '-----Original Message-----') && (trim($l) != "-----Message d'origine-----")) {
			$lines2[] = $l;
		}
		else {
			break;
		};
	};
	// ending with a blockquote? mmm smells like jeopardy to me
	if (count($lines2) > 0) {
			if (strstr($lines2[count($lines2) - 1], '</blockquote>')) {
				while (!strstr(array_pop($lines2), '<blockquote>'));
			};
	};
	// remove trailing blank lines
	if (count($lines2) > 0) {
		for ($i = count($lines2) - 1; $i >= 0; $i--) {
			if (trim($lines2[$i]) == '') {
				array_pop($lines2);
			}
			else {
				break;
			};
		};
	};
	// last line is a pre-quote? remove it.
	if (count($lines2) > 0) {
			if (preg_match('/:$/', $lines2[count($lines2) - 1])) {
				array_pop($lines2);
				// remove trailing blank lines again
				if (count($lines2) > 0) {
					for ($i = count($lines2) - 1; $i >= 0; $i--) {
						if (trim($lines2[$i]) == '') {
							array_pop($lines2);
						}
						else {
							break;
						};
					};
				};
			};
	};

	return $lines2;
}; 
// }}}
// function filter_body_to_xhtml_quotes($lines) {{{
/**
 * Wraps quotes in XHTML blockquotes
 * @param array $lines Lines of the message
 * @return array The cleaned up message
 */
function filter_body_to_xhtml_quotes($lines) {
	$lines2 = array();
	$numlines = count($lines);
	for ($i = 0; $i < $numlines; $i++) {
		$l = $lines[$i];
		$matches = array();
		if (preg_match("/^(&#62;)(.*)$/", $l, $matches)) {
			// this is where the quoting happens
			$j = $i;
			$quote_buffer = array();
			while (($j < $numlines) && (preg_match("/^((&#62;.*)|([[:space:]]*))$/", $lines[$j], $submatches))) {
				$quote_buffer[] = $lines[$j];
				$j++;
			};
			// remove empty quotes lines at the beginning
			while ((count($quote_buffer) > 0) && (trim(substr($quote_buffer[0], 5)) == '')) {
				$quote_buffer = array_slice($quote_buffer, 1);
			};
			// remove empty quotes lines at the end
			while ((count($quote_buffer) > 0) && (trim(substr($quote_buffer[count($quote_buffer) - 1], 5)) == ''))  {
				$quote_buffer = array_slice($quote_buffer, 0, count($quote_buffer) - 1);
			};
			// after removal of empty crap, is there anything left?
			if (count($quote_buffer) > 0) {
				$lines2[] = "<blockquote><div>";
				$lines2 = array_merge($lines2, $quote_buffer);
				$lines2[] = "";
				$lines2[] = "</div></blockquote>";
			};
			$i = ($j - 1);
		}
		else {
			// this is unquoted stuff, passthru
			$lines2[] = $l;
		};
	};

	return $lines2;
}; 
// }}}
// function filter_body_to_xhtml_signature($lines) {{{
/**
 * Wraps signature in XHTML div
 * Everything after the signature is discarded
 * @param array $lines Lines of the message
 * @return array The cleaned up message
 */
function filter_body_to_xhtml_signature($lines) {
	$lines2 = array();
	$numlines = count($lines);
	for ($i = 0; $i < $numlines; $i++) {
		$l = $lines[$i];
		$matches = array();
		if (preg_match("/^--[[:space:]]*$/", $l, $matches)) {
			$lines2[] = "<div class=\"sig\">";
			$j = $i;
			while ((isset($lines[$j])) && (trim($lines[$j]) != '')) {
		// This is ugly.  We gotta fix this otherwise.  Fix for
		// page http://ptaff.ca/ptafflist/2006-05/1148483917-clarete_et_referendum_(le_cas_du_montenegro)/
				$lines2[] .= strip_tags($lines[$j], '<img>');
				$j++;
			};
			$lines2[] = "</div>";
			// patch to return after sig
			return $lines2;
			// that next line was there before the patch
			//$i = ($j - 1);
		}
		else {
			$lines2[] = $l;
		};
	};
	return $lines2;
}; 
// }}}
// function filter_body_to_xhtml_empty_lines($lines) {{{
/**
 * Removes useless empty lines at the start of the message
 * @param array $lines Lines of the message
 * @return array The cleaned up message
 */
function filter_body_to_xhtml_empty_lines($lines) {
	if (XHTMAIL_DEBUG) { echo "\tEntering filter_body_to_xhtml_empty_lines\n"; };
	if (XHTMAIL_DEBUG) { echo "\t[" . trim($lines[0]) . "] (count:" . count($lines) . ")\n"; };
	while ((count($lines) > 0) && (trim($lines[0] == ''))) {
		$lines = array_slice($lines, 1);
	};
	return $lines;
}; 
// }}}
// function filter_body_to_xhtml_links($lines) {{{
/**
 * Wraps hyperlinks in XHTML a
 * @param array $lines Lines of the message
 * @return array The cleaned up message
 */
function filter_body_to_xhtml_links($lines) {
	// links
	$lines2 = array();
	foreach ($lines as $l) {
		$matches = array();
		preg_match("/^(.*([[:space:]]+))?(\<)?([[:alpha:]]+:\/\/[^<>[:space:]]+[-=[:alnum:]\/])(.*)$/", $l, $matches);
		if (count($matches)) {
			$pre = escape_xhtml_and_mail($matches[1] . $matches[3]);
			$uri = myurlencode(escape_xhtml_and_mail($matches[4], FALSE));
			$text = $matches[4];
			$text = escape_xhtml_and_mail($text);
			$post = escape_xhtml_and_mail($matches[5]);
			$lines2[] = "$pre<a href=\"$uri\">$text</a>$post";
		}
		else {
			$lines2[] = escape_xhtml_and_mail($l);
		};
	};
	return $lines2;
}; 
// }}}
// function filter_email_address($str) {{{
/**
 * Replaces e-mail addresses with something less spammer-friendly
 * @param string $str A string
 * @return string A filtered string
 */
function filter_email_address($str) {
	return preg_replace("/([[:alnum:]])@([[:alnum:]])/", "\\1<img class=\"arobas\" src=\"" . XHTMAIL_IMAGES_BASE_URI . "arobas.png\" alt=\"(&#224;)\" />\\2", $str); 
}; 
// }}}
// function filter_contributors($arr) {{{
/**
 * Removes duplicates from a contributor list
 * Contributor hash format:
 * <ul>
 * <li><var>['name'] </var> Contributor name</li>
 * <li><var>['x-url']</var> Contributor URL (if available)</li>
 * </ul>
 * @param array An array of contributor hashes
 * @return array A filtered array of contributor hashes
 */
function filter_contributors($arr) {
	// we'll remove right-part of @ in e-mails
	$outarray = array();
	$inarray = $arr;
	$used_names = array();
	if (is_array($inarray)) {
		foreach ($inarray as $i) {
			if ((!in_array($i['name'], $used_names)) && ($i['name'] != '')) {
				// we *might* have a HTMLized author name at that point.
				if (($pos = strpos($i['name'], "<img")) !== FALSE) {
					$i['name'] = substr($i['name'], 0, $pos);
				};
				$tmpoutarray = array();
				$tmpoutarray['name'] = $i['name'];
				$tmpoutarray['x-url'] = $i['x-url'];
				$outarray[] = $tmpoutarray;
				$used_names[] = $i['name'];
			};
		};
	};
	return $outarray;
}; 
// }}}
// function parse_template($str, $data) {{{
/**
 * Replaces tokens in the format <^FOO^> with their associated value in a string
 * @param string $str The original string to transform
 * @param array $data Hash of key-value pairs; example: array("FOO" => "bar") will replace occurences of <^FOO^> with bar
 * @return string The string with tokens replaced
 */
function parse_template($str, $data) {
	$lines = explode("\n", $str);
	$output = "";
	foreach ($lines as $l) {
		$tmp = $l;
		foreach ($data as $key => $value) {
			$tmp = preg_replace("/\<\^$key\^\>/", $data[$key], $tmp);
		};
		$output .= $tmp . "\n";
	};
	return $output;
}; 
// }}}
// function decode_header_to_utf8($str) {{{
/**
 * Transforms a header string to UTF-8 encoding
 * @param string $str String to encode
 * @return string The UTF-8 encoded string
 */
function decode_header_to_utf8($str) { 
	$newstr = Mail_mimeDecode::_decodeHeader($str);
	// check if string was changed; if so, check the encoding and reencode to UTF-8
	if ($str != $newstr) {
		$newstr = mail_encoding_to_utf8($newstr, $str, TRUE);
	};
	if (!is_utf8($newstr)) {
		// try to recover
		$newstr = utf8_encode($newstr);
	};
	return $newstr;
}; 
// }}}
// function index_build($files) {{{
/**
 * Builds the internal index for mail messages
 * Index hash format:
 * <ul>
 * <li><var>['filename']   </var> Filename</li>
 * <li><var>['id']		 </var> Message-ID</li>
 * <li><var>['subject']	</var> Subject</li>
 * <li><var>['in-reply-to']</var> In-Reply-To Message-ID</li>
 * <li><var>['unixdate']   </var> Unix timestamp of the message</li>
 * <li><var>['author']	 </var> Author</li>
 * <li><var>['child']	  </var> Children hash of this index entry, in the same format</li>
 * </ul>
 * @param array $files List of filenames to parse
 * @return array A hash with index info as described above
 */
function index_build($files) { 
	$index = array();
	// first, open all files, to fetch metainfo.
	$files_with_metadata = array();
	foreach ($files as $f) {
		if (XHTMAIL_DEBUG) { echo "Indexing $f\n"; };
		$mime = @mailparse_msg_parse_file($f);
		if (!is_resource($mime)) {
			die("File $f cannot be parsed by the mailparse library.  Aborting.\n");
		};
		$data = mailparse_msg_get_part_data($mime);
		$id = $data['headers']['message-id'];
	if (isset($data['headers']['subject'])) {
		$subject = trim(decode_header_to_utf8($data['headers']['subject']));
	}
	else {
		$subject = '';
	};
	if ($subject == '') {
		$subject = t('Untitled');
	};

		$in_reply_to = (isset($data['headers']['in-reply-to']) ? $data['headers']['in-reply-to'] : '');
		// patch in-reply-to with references
		if ($in_reply_to == '') {
			if ((isset($data['headers']['references'])) && ($data['headers']['references'] != '')) {
				$in_reply_to = substr(strrchr($data['headers']['references'], " "), 1);
			};
		};
		$date = $data['headers']['date'];
		$unixdate = strtotime($date);
		$author = decode_header_to_utf8($data['headers']['from']);
		$files_with_metadata[] = array("filename" => $f, "id" => $id, "subject" => $subject, "in-reply-to" => $in_reply_to, "unixdate" => $unixdate, "author" => $author);
	};
	// sort by date.  threading will be *much* easier afterwards!
	usort($files_with_metadata, 'sort_files_with_metadata');
	foreach ($files_with_metadata as $f) {
		if ($f['in-reply-to'] == '') { // new thread start
			$index[] = $f;
		}
		else { // thread seems to exist.
			// is there a living parent?
			$found_parent = FALSE;
			foreach ($files_with_metadata as $g) {
				if ($g['id'] == $f['in-reply-to']) {
					$found_parent = TRUE;
					index_insert_child($index, $f);
					break;
				};
			};
			if (!$found_parent) { // oh, oh!
				$index[] = $f;
			};
		};

	};
	return $index;
}; 
// }}}
// function index_insert_child(&$index, $fwm) {{{
/**
 * Adds a children to the main index
 * @param array &$index The main index
 * @param array $fwm The child to add
 * @return boolean TRUE if the child was inserted, FALSE if there were an error
 * @see function index_build
 */
function index_insert_child(&$index, $fwm) {
	// scan all $index.  when meeting a subnode, recurse
	$index_length = count($index);
	for ($i = 0; $i < $index_length; $i++) {
		if ($index[$i]['id'] == $fwm['in-reply-to']) { // immediate child
			$index[$i]['child'][] = $fwm;
			return TRUE;
		};
		// check if there are children
		if ((isset($index[$i]['child'])) && (count($index[$i]['child']))) {
			if (index_insert_child($index[$i]['child'], $fwm)) {
				return TRUE;
			};
		};
	};
	return FALSE;  
}; 
// }}}
// function sort_files_with_metadata($a, $b) {{{
/**
 * Callback function to sort 2 hashes by their ['unixdate'] key
 * @param array $a Hash containing a ['unixdate'] key
 * @param array $b Hash containing a ['unixdate'] key
 * @return integer -1 if $a is smaller, 0 if they are equal, 1 if $b is smaller
 */
function sort_files_with_metadata($a, $b) {
	if ($a['unixdate'] < $b['unixdate']) { return -1; };
	if ($a['unixdate'] > $b['unixdate']) { return 1; };
	return 0;
}; 
// }}}
// function count_msg_in_thread($index_entry) {{{
/**
 * Counts the number of messages in a thread
 * @param array $index_entry An index entry
 * @return integer The number of messages in the thread
 * @see function index_build
 */
function count_msg_in_thread($index_entry) { 
	$count = 1;
	if ((isset($index_entry['child'])) && (is_array($index_entry['child']))) {
		foreach ($index_entry['child'] as $i) {
			$count += count_msg_in_thread($i);
		};
	};
	return $count;
}; 
// }}}
// function encode_utf8_filename_to_ascii($str) {{{
/**
 * Replaces a string with something suitable for a filename
 * @param string $str The UTF-8 string
 * @returns string The encoded string
 */
function encode_utf8_filename_to_ascii($str) {
	static $from;
	static $to;
	if (!is_array($from)) {
		$from = array("/", " ", "<", ">", "#",  "?", "|", "%", "&", "'", '"');
		$to = array  ("-", "_", "_", "_", "no", "",  "",  "",  "",  "",  "");
	};
	$str = strtolower(remove_accents(str_replace($from, $to, $str)));
	return $str;
}; 
// }}}
// function remove_accents($str) {{{
/**
 * Flaky replacement for iconv's TRANSLIT
 * Replaces accented and foreign characters with some reasonable ASCII-7 characters
 * @param string $str The string to translit
 * @return string The translited string
 */
function remove_accents($str) {
	// the TRANSLIT option of iconv doesn't seem to work here.  
	// We must get some work done... so use this ugly crap meanwhile
	$trans["A"] = array(192, 193, 194, 195, 196, 197, 198);
	$trans["C"] = array(199);
	$trans["E"] = array(200, 201, 202, 203);
	$trans["I"] = array(204, 205, 206, 207);
	$trans["N"] = array(209);
	$trans["O"] = array(210, 211, 212, 213, 214, 216);
	$trans["U"] = array(217, 218, 219, 220);
	$trans["Y"] = array(221);
	$trans["a"] = array(224, 225, 226, 227, 228, 229, 230);
	$trans["c"] = array(231);
	$trans["e"] = array(232, 233, 234, 235);
	$trans["i"] = array(236, 237, 238, 239);
	$trans["n"] = array(241);
	$trans["o"] = array(242, 243, 244, 245, 246, 248);
	$trans["u"] = array(249, 250, 251, 252);
	$trans["y"] = array(253, 255);
	$tmpstr = utf8_decode($str);
	foreach ($trans as $new => $array_of_chars) {
		foreach ($array_of_chars as $a) {
			$tmpstr = str_replace(chr($a), $new, $tmpstr);
		};
	};
	// remove further crap
	$outstr = '';
	for ($i = 0; $i < strlen($tmpstr); $i++) {
		$outstr .= (((ord($tmpstr[$i]) > 32) && (ord($tmpstr[$i]) < 123)) ? $tmpstr[$i] : '_');
	};
	return $outstr;
}; 
// }}}
// function get_country_name_from_tld($tld) {{{
/**
 * Get a country name from a top level domain
 * @param string $tld Top-level domain
 * @return string The full-length country name
 */
function get_country_name_from_tld($tld) {
	static $set;
	if (!is_array($set)) {
		$set = array(
			"ac" => t("Ascension Island"),
			"ad" => t("Andorra"),
			"aero" => t("Aero/Travel domains"),
			"ae" => t("United Arab Emirates"),
			"af" => t("Afghanistan"),
			"ag" => t("Antigua and Barbuda"),
			"ai" => t("Anguilla"),
			"al" => t("Albania"),
			"am" => t("Armenia"),
			"an" => t("Netherlands Antilles"),
			"ao" => t("Angola"),
			"aq" => t("Antarctica"),
			"ar" => t("Argentina"),
			"arpa" => t("Old style Arpanet"),
			"as" => t("American Samoa"),
			"at" => t("Austria"),
			"au" => t("Australia"),
			"aw" => t("Aruba"),
			"az" => t("Azerbaidjan"),
			"ba" => t("Bosnia-Herzegovina"),
			"bb" => t("Barbados"),
			"bd" => t("Bangladesh"),
			"be" => t("Belgium"),
			"bf" => t("Burkina Faso"),
			"bg" => t("Bulgaria"),
			"bh" => t("Bahrain"),
			"bi" => t("Burundi"),
			"biz" => t("Biz domains"),
			"bj" => t("Benin"),
			"bm" => t("Bermuda"),
			"bn" => t("Brunei Darussalam"),
			"bo" => t("Bolivia"),
			"br" => t("Brazil"),
			"bs" => t("Bahamas"),
			"bt" => t("Bhutan"),
			"bv" => t("Bouvet Island"),
			"bw" => t("Botswana"),
			"by" => t("Belarus"),
			"bz" => t("Belize"),
			"ca" => t("Canada"),
			"cc" => t("Cocos (Keeling) Islands"),
			"cd" => t("Congo, Democratic Republic of the"),
			"cf" => t("Central African Republic"),
			"cg" => t("Congo"),
			"ch" => t("Switzerland"),
			"ci" => t("Ivory Coast (Cote D'Ivoire)"),
			"ck" => t("Cook Islands"),
			"cl" => t("Chile"),
			"cm" => t("Cameroon"),
			"cn" => t("China"),
			"co" => t("Colombia"),
			"com" => t("Commercial"),
			"coop" => t("Coop domains"),
			"cr" => t("Costa Rica"),
			"cu" => t("Cuba"),
			"cv" => t("Cape Verde"),
			"cx" => t("Christmas Island"),
			"cy" => t("Cyprus"),
			"cz" => t("Czech Republic"),
			"de" => t("Germany"),
			"dj" => t("Djibouti"),
			"dk" => t("Denmark"),
			"dm" => t("Dominica"),
			"do" => t("Dominican Republic"),
			"dz" => t("Algeria"),
			"ec" => t("Ecuador"),
			"edu" => t("USA Educational"),
			"ee" => t("Estonia"),
			"eg" => t("Egypt"),
			"eh" => t("Western Sahara"),
			"er" => t("Eritrea"),
			"es" => t("Spain"),
			"et" => t("Ethiopia"),
			"fi" => t("Finland"),
			"fj" => t("Fiji"),
			"fk" => t("Falkland Islands"),
			"fm" => t("Micronesia"),
			"fo" => t("Faroe Islands"),
			"fr" => t("France"),
			"ga" => t("Gabon"),
			"gd" => t("Grenada"),
			"ge" => t("Georgia"),
			"gf" => t("French Guyana"),
			"gg" => t("Guernsey"),
			"gh" => t("Ghana"),
			"gi" => t("Gibraltar"),
			"gl" => t("Greenland"),
			"gm" => t("Gambia"),
			"gn" => t("Guinea"),
			"gov" => t("USA Government"),
			"gp" => t("Guadeloupe (French)"),
			"gq" => t("Equatorial Guinea"),
			"gr" => t("Greece"),
			"gs" => t("S. Georgia &#38; S. Sandwich Isls."),
			"gt" => t("Guatemala"),
			"gu" => t("Guam (USA)"),
			"gw" => t("Guinea Bissau"),
			"gy" => t("Guyana"),
			"hk" => t("Hong Kong"),
			"hm" => t("Heard and McDonald Islands"),
			"hn" => t("Honduras"),
			"hr" => t("Croatia"),
			"ht" => t("Haiti"),
			"hu" => t("Hungary"),
			"id" => t("Indonesia"),
			"ie" => t("Ireland"),
			"il" => t("Israel"),
			"im" => t("Isle of Man"),
			"info" => t("Info domains"),
			"in" => t("India"),
			"int" => t("International"),
			"io" => t("British Indian Ocean Territory"),
			"iq" => t("Iraq"),
			"ir" => t("Iran"),
			"is" => t("Iceland"),
			"it" => t("Italy"),
			"je" => t("Jersey"),
			"jm" => t("Jamaica"),
			"jo" => t("Jordan"),
			"jp" => t("Japan"),
			"ke" => t("Kenya"),
			"kg" => t("Kyrgyzstan"),
			"kh" => t("Cambodia"),
			"ki" => t("Kiribati"),
			"km" => t("Comoros"),
			"kn" => t("Saint Kitts &#38; Nevis Anguilla"),
			"kp" => t("North Korea"),
			"kr" => t("South Korea"),
			"kw" => t("Kuwait"),
			"ky" => t("Cayman Islands"),
			"kz" => t("Kazakhstan"),
			"la" => t("Laos"),
			"lb" => t("Lebanon"),
			"lc" => t("Saint Lucia"),
			"li" => t("Liechtenstein"),
			"lk" => t("Sri Lanka"),
			"localhost" => t("localhost"),
			"lr" => t("Liberia"),
			"ls" => t("Lesotho"),
			"lt" => t("Lithuania"),
			"lu" => t("Luxembourg"),
			"lv" => t("Latvia"),
			"ly" => t("Libya"),
			"ma" => t("Morocco"),
			"mc" => t("Monaco"),
			"md" => t("Moldova"),
			"mg" => t("Madagascar"),
			"mh" => t("Marshall Islands"),
			"mil" => t("USA Military"),
			"mk" => t("Macedonia"),
			"ml" => t("Mali"),
			"mm" => t("Myanmar"),
			"mn" => t("Mongolia"),
			"mo" => t("Macau"),
			"mp" => t("Northern Mariana Islands"),
			"mq" => t("Martinique (French)"),
			"mr" => t("Mauritania"),
			"ms" => t("Montserrat"),
			"mt" => t("Malta"),
			"mu" => t("Mauritius"),
			"museum" => t("Museum domains"),
			"mv" => t("Maldives"),
			"mw" => t("Malawi"),
			"mx" => t("Mexico"),
			"my" => t("Malaysia"),
			"mz" => t("Mozambique"),
			"name" => t("Name domains"),
			"na" => t("Namibia"),
			"nato" => t("NATO"),
			"nc" => t("New Caledonia (French)"),
			"ne" => t("Niger"),
			"net" => t("Network"),
			"nf" => t("Norfolk Island"),
			"ng" => t("Nigeria"),
			"ni" => t("Nicaragua"),
			"nl" => t("Netherlands"),
			"no" => t("Norway"),
			"np" => t("Nepal"),
			"nr" => t("Nauru"),
			"nu" => t("Niue"),
			"nz" => t("New Zealand"),
			"om" => t("Oman"),
			"org" => t("Non-Profit Organizations"),
			"pa" => t("Panama"),
			"pe" => t("Peru"),
			"pf" => t("Polynesia (French)"),
			"pg" => t("Papua New Guinea"),
			"ph" => t("Philippines"),
			"pk" => t("Pakistan"),
			"pl" => t("Poland"),
			"pm" => t("Saint Pierre and Miquelon"),
			"pn" => t("Pitcairn Island"),
			"pro" => t("Professional domains"),
			"pr" => t("Puerto Rico"),
			"ps" => t("Palestinian Territories"),
			"pt" => t("Portugal"),
			"pw" => t("Palau"),
			"py" => t("Paraguay"),
			"qa" => t("Qatar"),
			"re" => t("Reunion (French)"),
			"ro" => t("Romania"),
			"ru" => t("Russian Federation"),
			"rw" => t("Rwanda"),
			"sa" => t("Saudi Arabia"),
			"sb" => t("Solomon Islands"),
			"sc" => t("Seychelles"),
			"sd" => t("Sudan"),
			"se" => t("Sweden"),
			"sg" => t("Singapore"),
			"sh" => t("Saint Helena"),
			"si" => t("Slovenia"),
			"sj" => t("Svalbard and Jan Mayen Islands"),
			"sk" => t("Slovak Republic"),
			"sl" => t("Sierra Leone"),
			"sm" => t("San Marino"),
			"sn" => t("Senegal"),
			"so" => t("Somalia"),
			"sr" => t("Suriname"),
			"st" => t("Sao Tome and Principe"),
			"su" => t("Soviet Union"),
			"sv" => t("El Salvador"),
			"sy" => t("Syria"),
			"sz" => t("Swaziland"),
			"tc" => t("Turks and Caicos Islands"),
			"td" => t("Chad"),
			"tf" => t("French Southern Territories"),
			"tg" => t("Togo"),
			"th" => t("Thailand"),
			"tj" => t("Tajikistan"),
			"tk" => t("Tokelau"),
			"tm" => t("Turkmenistan"),
			"tn" => t("Tunisia"),
			"to" => t("Tonga"),
			"tp" => t("East Timor"),
			"tr" => t("Turkey"),
			"tt" => t("Trinidad and Tobago"),
			"tv" => t("Tuvalu"),
			"tw" => t("Taiwan"),
			"tz" => t("Tanzania"),
			"ua" => t("Ukraine"),
			"ug" => t("Uganda"),
			"uk" => t("United Kingdom"),
			"um" => t("USA Minor Outlying Islands"),
			"us" => t("United States"),
			"uy" => t("Uruguay"),
			"uz" => t("Uzbekistan"),
			"va" => t("Vatican City State"),
			"vc" => t("Saint Vincent &#38; Grenadines"),
			"ve" => t("Venezuela"),
			"vg" => t("Virgin Islands (British)"),
			"vi" => t("Virgin Islands (USA)"),
			"vn" => t("Vietnam"),
			"vu" => t("Vanuatu"),
			"wf" => t("Wallis and Futuna Islands"),
			"ws" => t("Samoa"),
			"ye" => t("Yemen"),
			"yt" => t("Mayotte"),
			"yu" => t("Serbia and Montenegro"),
			"za" => t("South Africa"),
			"zm" => t("Zambia"),
			"zw" => t("Zimbabwe")
		);
	};
	return $set[$tld];
}; 
// }}}
// function get_country_uri_from_tld($tld) {{{
/**
 * Get a country URI from a top level domain
 * @param string $tld Top-level domain
 * @return string URI to a page describing the country
 */
function get_country_uri_from_tld($tld) {
	static $set;
	if (!is_array($set)) {
		$set = array(
			"ac" => t("http://en.wikipedia.org/wiki/Ascension_Island"),
			"ad" => t("http://en.wikipedia.org/wiki/Andorra"),
			"aero" => t("http://en.wikipedia.org/wiki/.aero"),
			"ae" => t("http://en.wikipedia.org/wiki/United_Arab_Emirates"),
			"af" => t("http://en.wikipedia.org/wiki/Afghanistan"),
			"ag" => t("http://en.wikipedia.org/wiki/Antigua_and_Barbuda"),
			"ai" => t("http://en.wikipedia.org/wiki/Anguilla"),
			"al" => t("http://en.wikipedia.org/wiki/Albania"),
			"am" => t("http://en.wikipedia.org/wiki/Armenia"),
			"an" => t("http://en.wikipedia.org/wiki/Netherlands_Antilles"),
			"ao" => t("http://en.wikipedia.org/wiki/Angola"),
			"aq" => t("http://en.wikipedia.org/wiki/Antarctica"),
			"ar" => t("http://en.wikipedia.org/wiki/Argentina"),
			"arpa" => t("http://en.wikipedia.org/wiki/ARPANET"),
			"as" => t("http://en.wikipedia.org/wiki/American_Samoa"),
			"at" => t("http://en.wikipedia.org/wiki/Austria"),
			"au" => t("http://en.wikipedia.org/wiki/Australia"),
			"aw" => t("http://en.wikipedia.org/wiki/Aruba"),
			"az" => t("http://en.wikipedia.org/wiki/Azerbaijan"),
			"ba" => t("http://en.wikipedia.org/wiki/Bosnia_and_Herzegovina"),
			"bb" => t("http://en.wikipedia.org/wiki/Barbados"),
			"bd" => t("http://en.wikipedia.org/wiki/Bangladesh"),
			"be" => t("http://en.wikipedia.org/wiki/Belgium"),
			"bf" => t("http://en.wikipedia.org/wiki/Burkina_Faso"),
			"bg" => t("http://en.wikipedia.org/wiki/Bulgaria"),
			"bh" => t("http://en.wikipedia.org/wiki/Bahrain"),
			"bi" => t("http://en.wikipedia.org/wiki/Burundi"),
			"biz" => t("http://en.wikipedia.org/wiki/.biz"),
			"bj" => t("http://en.wikipedia.org/wiki/Benin"),
			"bm" => t("http://en.wikipedia.org/wiki/Bermuda"),
			"bn" => t("http://en.wikipedia.org/wiki/Brunei"),
			"bo" => t("http://en.wikipedia.org/wiki/Bolivia"),
			"br" => t("http://en.wikipedia.org/wiki/Brazil"),
			"bs" => t("http://en.wikipedia.org/wiki/Bahamas"),
			"bt" => t("http://en.wikipedia.org/wiki/Bhutan"),
			"bv" => t("http://en.wikipedia.org/wiki/Bouvet_Island"),
			"bw" => t("http://en.wikipedia.org/wiki/Botswana"),
			"by" => t("http://en.wikipedia.org/wiki/Belarus"),
			"bz" => t("http://en.wikipedia.org/wiki/Belize"),
			"ca" => t("http://en.wikipedia.org/wiki/Canada"),
			"cc" => t("http://en.wikipedia.org/wiki/Cocos_Island"),
			"cd" => t("http://en.wikipedia.org/wiki/Democratic_Republic_of_the_Congo"),
			"cf" => t("http://en.wikipedia.org/wiki/Central_African_Republic"),
			"cg" => t("http://en.wikipedia.org/wiki/Republic_of_the_Congo"),
			"ch" => t("http://en.wikipedia.org/wiki/Switzerland"),
			"ci" => t("http://en.wikipedia.org/wiki/C%F4ote_d'Ivoire"),
			"ck" => t("http://en.wikipedia.org/wiki/Cook_Islands"),
			"cl" => t("http://en.wikipedia.org/wiki/Chile"),
			"cm" => t("http://en.wikipedia.org/wiki/Cameroon"),
			"cn" => t("http://en.wikipedia.org/wiki/People's_Republic_of_China"),
			"co" => t("http://en.wikipedia.org/wiki/Colombia"),
			"com" => t("http://en.wikipedia.org/wiki/.com"),
			"coop" => t("http://en.wikipedia.org/wiki/.coop"),
			"cr" => t("http://en.wikipedia.org/wiki/Costa_Rica"),
			"cu" => t("http://en.wikipedia.org/wiki/Cuba"),
			"cv" => t("http://en.wikipedia.org/wiki/Cape_Verde"),
			"cx" => t("http://en.wikipedia.org/wiki/Christmas_Island"),
			"cy" => t("http://en.wikipedia.org/wiki/Cyprus"),
			"cz" => t("http://en.wikipedia.org/wiki/Czech_Republic"),
			"de" => t("http://en.wikipedia.org/wiki/Germany"),
			"dj" => t("http://en.wikipedia.org/wiki/Djibouti"),
			"dk" => t("http://en.wikipedia.org/wiki/Denmark"),
			"dm" => t("http://en.wikipedia.org/wiki/Dominica"),
			"do" => t("http://en.wikipedia.org/wiki/Dominican_Republic"),
			"dz" => t("http://en.wikipedia.org/wiki/Algeria"),
			"ec" => t("http://en.wikipedia.org/wiki/Ecuador"),
			"edu" => t("http://en.wikipedia.org/wiki/.edu"),
			"ee" => t("http://en.wikipedia.org/wiki/Estonia"),
			"eg" => t("http://en.wikipedia.org/wiki/Egypt"),
			"eh" => t("http://en.wikipedia.org/wiki/Western_Sahara"),
			"er" => t("http://en.wikipedia.org/wiki/Eritrea"),
			"es" => t("http://en.wikipedia.org/wiki/Spain"),
			"et" => t("http://en.wikipedia.org/wiki/Ethiopia"),
			"fi" => t("http://en.wikipedia.org/wiki/Finland"),
			"fj" => t("http://en.wikipedia.org/wiki/Fiji"),
			"fk" => t("http://en.wikipedia.org/wiki/Falkland_Islands"),
			"fm" => t("http://en.wikipedia.org/wiki/Federated_States_of_Micronesia"),
			"fo" => t("http://en.wikipedia.org/wiki/Faroe_Islands"),
			"fr" => t("http://en.wikipedia.org/wiki/France"),
			"ga" => t("http://en.wikipedia.org/wiki/Gabon"),
			"gd" => t("http://en.wikipedia.org/wiki/Grenada"),
			"ge" => t("http://en.wikipedia.org/wiki/Georgia_(country)"),
			"gf" => t("http://en.wikipedia.org/wiki/French_Guiana"),
			"gg" => t("http://en.wikipedia.org/wiki/Guernsey"),
			"gh" => t("http://en.wikipedia.org/wiki/Ghana"),
			"gi" => t("http://en.wikipedia.org/wiki/Gibraltar"),
			"gl" => t("http://en.wikipedia.org/wiki/Greenland"),
			"gm" => t("http://en.wikipedia.org/wiki/The_Gambia"),
			"gn" => t("http://en.wikipedia.org/wiki/Guinea"),
			"gov" => t("http://en.wikipedia.org/wiki/.gov"),
			"gp" => t("http://en.wikipedia.org/wiki/Guadeloupe"),
			"gq" => t("http://en.wikipedia.org/wiki/Equatorial_Guinea"),
			"gr" => t("http://en.wikipedia.org/wiki/Greece"),
			"gs" => t("http://en.wikipedia.org/wiki/South_Georgia_and_the_South_Sandwich_Islands"),
			"gt" => t("http://en.wikipedia.org/wiki/Guatemala"),
			"gu" => t("http://en.wikipedia.org/wiki/Guam"),
			"gw" => t("http://en.wikipedia.org/wiki/Guinea-Bissau"),
			"gy" => t("http://en.wikipedia.org/wiki/Guyana"),
			"hk" => t("http://en.wikipedia.org/wiki/Hong_Kong"),
			"hm" => t("http://en.wikipedia.org/wiki/Heard_Island_and_McDonald_Islands"),
			"hn" => t("http://en.wikipedia.org/wiki/Honduras"),
			"hr" => t("http://en.wikipedia.org/wiki/Croatia"),
			"ht" => t("http://en.wikipedia.org/wiki/Haiti"),
			"hu" => t("http://en.wikipedia.org/wiki/Hungary"),
			"id" => t("http://en.wikipedia.org/wiki/Indonesia"),
			"ie" => t("http://en.wikipedia.org/wiki/Republic_of_Ireland"),
			"il" => t("http://en.wikipedia.org/wiki/Israel"),
			"im" => t("http://en.wikipedia.org/wiki/Isle_of_Man"),
			"info" => t("http://en.wikipedia.org/wiki/.info"),
			"in" => t("http://en.wikipedia.org/wiki/India"),
			"int" => t("http://en.wikipedia.org/wiki/.int"),
			"io" => t("http://en.wikipedia.org/wiki/British_Indian_Ocean_Territory"),
			"iq" => t("http://en.wikipedia.org/wiki/Iraq"),
			"ir" => t("http://en.wikipedia.org/wiki/Iran"),
			"is" => t("http://en.wikipedia.org/wiki/Iceland"),
			"it" => t("http://en.wikipedia.org/wiki/Italy"),
			"je" => t("http://en.wikipedia.org/wiki/Jersey"),
			"jm" => t("http://en.wikipedia.org/wiki/Jamaica"),
			"jo" => t("http://en.wikipedia.org/wiki/Jordan"),
			"jp" => t("http://en.wikipedia.org/wiki/Japan"),
			"ke" => t("http://en.wikipedia.org/wiki/Kenya"),
			"kg" => t("http://en.wikipedia.org/wiki/Kyrgyzstan"),
			"kh" => t("http://en.wikipedia.org/wiki/Cambodia"),
			"ki" => t("http://en.wikipedia.org/wiki/Kiribati"),
			"km" => t("http://en.wikipedia.org/wiki/Comoros"),
			"kn" => t("http://en.wikipedia.org/wiki/Saint_Kitts_and_Nevis"),
			"kp" => t("http://en.wikipedia.org/wiki/North_Korea"),
			"kr" => t("http://en.wikipedia.org/wiki/South_Korea"),
			"kw" => t("http://en.wikipedia.org/wiki/Kuwait"),
			"ky" => t("http://en.wikipedia.org/wiki/Cayman_Islands"),
			"kz" => t("http://en.wikipedia.org/wiki/Kazakhstan"),
			"la" => t("http://en.wikipedia.org/wiki/Laos"),
			"lb" => t("http://en.wikipedia.org/wiki/Lebanon"),
			"lc" => t("http://en.wikipedia.org/wiki/Saint_Lucia"),
			"li" => t("http://en.wikipedia.org/wiki/Liechtenstein"),
			"lk" => t("http://en.wikipedia.org/wiki/Sri_Lanka"),
			"localhost" => t("http://localhost/"),
			"lr" => t("http://en.wikipedia.org/wiki/Liberia"),
			"ls" => t("http://en.wikipedia.org/wiki/Lesotho"),
			"lt" => t("http://en.wikipedia.org/wiki/Lithuania"),
			"lu" => t("http://en.wikipedia.org/wiki/Luxembourg"),
			"lv" => t("http://en.wikipedia.org/wiki/Latvia"),
			"ly" => t("http://en.wikipedia.org/wiki/Libya"),
			"ma" => t("http://en.wikipedia.org/wiki/Morocco"),
			"mc" => t("http://en.wikipedia.org/wiki/Monaco"),
			"md" => t("http://en.wikipedia.org/wiki/Moldova"),
			"mg" => t("http://en.wikipedia.org/wiki/Madagascar"),
			"mh" => t("http://en.wikipedia.org/wiki/Marshall_Islands"),
			"mil" => t("http://en.wikipedia.org/wiki/.mil"),
			"mk" => t("http://en.wikipedia.org/wiki/Republic_of_Macedonia"),
			"ml" => t("http://en.wikipedia.org/wiki/Mali"),
			"mm" => t("http://en.wikipedia.org/wiki/Myanmar"),
			"mn" => t("http://en.wikipedia.org/wiki/Mongolia_(country)"),
			"mo" => t("http://en.wikipedia.org/wiki/Macau"),
			"mp" => t("http://en.wikipedia.org/wiki/Northern_Mariana_Islands"),
			"mq" => t("http://en.wikipedia.org/wiki/Martinique"),
			"mr" => t("http://en.wikipedia.org/wiki/Mauritania"),
			"ms" => t("http://en.wikipedia.org/wiki/Montserrat"),
			"mt" => t("http://en.wikipedia.org/wiki/Malta"),
			"mu" => t("http://en.wikipedia.org/wiki/Mauritius"),
			"museum" => t("http://en.wikipedia.org/wiki/.museum"),
			"mv" => t("http://en.wikipedia.org/wiki/Maldives"),
			"mw" => t("http://en.wikipedia.org/wiki/Malawi"),
			"mx" => t("http://en.wikipedia.org/wiki/Mexico"),
			"my" => t("http://en.wikipedia.org/wiki/Malaysia"),
			"mz" => t("http://en.wikipedia.org/wiki/Mozambique"),
			"name" => t("http://en.wikipedia.org/wiki/.name"),
			"na" => t("http://en.wikipedia.org/wiki/Namibia"),
			"nato" => t("http://en.wikipedia.org/wiki/NATO"),
			"nc" => t("http://en.wikipedia.org/wiki/New_Caledonia"),
			"ne" => t("http://en.wikipedia.org/wiki/Niger"),
			"net" => t("http://en.wikipedia.org/wiki/.net"),
			"nf" => t("http://en.wikipedia.org/wiki/Norfolk_Island"),
			"ng" => t("http://en.wikipedia.org/wiki/Nigeria"),
			"ni" => t("http://en.wikipedia.org/wiki/Nicaragua"),
			"nl" => t("http://en.wikipedia.org/wiki/Netherlands"),
			"no" => t("http://en.wikipedia.org/wiki/Norway"),
			"np" => t("http://en.wikipedia.org/wiki/Nepal"),
			"nr" => t("http://en.wikipedia.org/wiki/Nauru"),
			"nu" => t("http://en.wikipedia.org/wiki/Niue"),
			"nz" => t("http://en.wikipedia.org/wiki/New_Zealand"),
			"om" => t("http://en.wikipedia.org/wiki/Oman"),
			"org" => t("http://en.wikipedia.org/wiki/.org"),
			"pa" => t("http://en.wikipedia.org/wiki/Panama"),
			"pe" => t("http://en.wikipedia.org/wiki/Peru"),
			"pf" => t("http://en.wikipedia.org/wiki/French_Polynesia"),
			"pg" => t("http://en.wikipedia.org/wiki/Papua_New_Guinea"),
			"ph" => t("http://en.wikipedia.org/wiki/Philippines"),
			"pk" => t("http://en.wikipedia.org/wiki/Pakistan"),
			"pl" => t("http://en.wikipedia.org/wiki/Poland"),
			"pm" => t("http://en.wikipedia.org/wiki/Saint_Pierre_and_Miquelon"),
			"pn" => t("http://en.wikipedia.org/wiki/Pitcairn_Islands"),
			"pro" => t("http://en.wikipedia.org/wiki/.pro"),
			"pr" => t("http://en.wikipedia.org/wiki/Puerto_Rico"),
			"ps" => t("http://en.wikipedia.org/wiki/State_of_Palestine"),
			"pt" => t("http://en.wikipedia.org/wiki/Portugal"),
			"pw" => t("http://en.wikipedia.org/wiki/Palau"),
			"py" => t("http://en.wikipedia.org/wiki/Paraguay"),
			"qa" => t("http://en.wikipedia.org/wiki/Qatar"),
			"re" => t("http://en.wikipedia.org/wiki/La_R%E9union"),
			"ro" => t("http://en.wikipedia.org/wiki/Romania"),
			"ru" => t("http://en.wikipedia.org/wiki/Russia"),
			"rw" => t("http://en.wikipedia.org/wiki/Rwanda"),
			"sa" => t("http://en.wikipedia.org/wiki/Saudir_Arabia"),
			"sb" => t("http://en.wikipedia.org/wiki/Solomon_Islands"),
			"sc" => t("http://en.wikipedia.org/wiki/Seychelles"),
			"sd" => t("http://en.wikipedia.org/wiki/Sudan"),
			"se" => t("http://en.wikipedia.org/wiki/Sweden"),
			"sg" => t("http://en.wikipedia.org/wiki/Singapore"),
			"sh" => t("http://en.wikipedia.org/wiki/St_Helena"),
			"si" => t("http://en.wikipedia.org/wiki/Slovenia"),
			"sj" => t("http://en.wikipedia.org/wiki/Svalbard"),
			"sk" => t("http://en.wikipedia.org/wiki/Slovakia"),
			"sl" => t("http://en.wikipedia.org/wiki/Sierra_Leone"),
			"sm" => t("http://en.wikipedia.org/wiki/San_Marino"),
			"sn" => t("http://en.wikipedia.org/wiki/Senegal"),
			"so" => t("http://en.wikipedia.org/wiki/Somalia"),
			"sr" => t("http://en.wikipedia.org/wiki/Suriname"),
			"st" => t("http://en.wikipedia.org/wiki/S%E3o_Tom%E9_and_Pr%EDncipe"),
			"su" => t("http://en.wikipedia.org/wiki/Soviet_Union"),
			"sv" => t("http://en.wikipedia.org/wiki/El_Salvador"),
			"sy" => t("http://en.wikipedia.org/wiki/Syria"),
			"sz" => t("http://en.wikipedia.org/wiki/Swaziland"),
			"tc" => t("http://en.wikipedia.org/wiki/Turks_and_Caicos_Islands"),
			"td" => t("http://en.wikipedia.org/wiki/Chad"),
			"tf" => t("http://en.wikipedia.org/wiki/French_Southern_Territories"),
			"tg" => t("http://en.wikipedia.org/wiki/Togo"),
			"th" => t("http://en.wikipedia.org/wiki/Thailand"),
			"tj" => t("http://en.wikipedia.org/wiki/Tajikistan"),
			"tk" => t("http://en.wikipedia.org/wiki/Tokelau"),
			"tm" => t("http://en.wikipedia.org/wiki/Turkmenistan"),
			"tn" => t("http://en.wikipedia.org/wiki/Tunisia"),
			"to" => t("http://en.wikipedia.org/wiki/Tonga"),
			"tp" => t("http://en.wikipedia.org/wiki/East_Timor"),
			"tr" => t("http://en.wikipedia.org/wiki/Turkey"),
			"tt" => t("http://en.wikipedia.org/wiki/Trinidad_and_Tobago"),
			"tv" => t("http://en.wikipedia.org/wiki/Tuvalu"),
			"tw" => t("http://en.wikipedia.org/wiki/Republic_of_China"),
			"tz" => t("http://en.wikipedia.org/wiki/Tanzania"),
			"ua" => t("http://en.wikipedia.org/wiki/Ukraine"),
			"ug" => t("http://en.wikipedia.org/wiki/Uganda"),
			"uk" => t("http://en.wikipedia.org/wiki/United_Kingdom"),
			"um" => t("http://en.wikipedia.org/wiki/United_States_Minor_Outlying_Islands"),
			"us" => t("http://en.wikipedia.org/wiki/United_States"),
			"uy" => t("http://en.wikipedia.org/wiki/Uruguay"),
			"uz" => t("http://en.wikipedia.org/wiki/Uzbekistan"),
			"va" => t("http://en.wikipedia.org/wiki/Vatican_City"),
			"vc" => t("http://en.wikipedia.org/wiki/Saint_Vincent_and_the_Grenadines"),
			"ve" => t("http://en.wikipedia.org/wiki/Venezuela"),
			"vg" => t("http://en.wikipedia.org/wiki/British_Virgin_Islands"),
			"vi" => t("http://en.wikipedia.org/wiki/U.S._Virgin_Islands"),
			"vn" => t("http://en.wikipedia.org/wiki/Vietnam"),
			"vu" => t("http://en.wikipedia.org/wiki/Vanuatu"),
			"wf" => t("http://en.wikipedia.org/wiki/Wallis_and_Futuna"),
			"ws" => t("http://en.wikipedia.org/wiki/Samoa"),
			"ye" => t("http://en.wikipedia.org/wiki/Yemen"),
			"yt" => t("http://en.wikipedia.org/wiki/Mayotte"),
			"yu" => t("http://en.wikipedia.org/wiki/Serbia_and_Montenegro"),
			"za" => t("http://en.wikipedia.org/wiki/South_Africa"),
			"zm" => t("http://en.wikipedia.org/wiki/Zambia"),
			"zw" => t("http://en.wikipedia.org/wiki/Zimbabwe")
		);
	};
	return $set[$tld];
}; 
// }}}
// function get_mua_uri($code) {{{
/**
 * Get a mail user agent URI from an internal code
 * @param string $code Internal MUA code
 * @return string URI to a page describing the mail user agent
 * @see function extract_mua_from_email_header
 */
function get_mua_uri($code) {
	static $uri;
	if (!is_array($uri)) {
		$uri = array();
		$uri['alpine']	 = t('http://en.wikipedia.org/wiki/Alpine_%28e-mail_client%29');
		$uri['eudora']		  = t('http://en.wikipedia.org/wiki/Eudora_%28email_client%29');
		$uri['evolution']	   = t('http://en.wikipedia.org/wiki/Novell_Evolution');
		$uri['google']		  = t('http://en.wikipedia.org/wiki/Gmail');
		$uri['imp']			 = t('http://en.wikipedia.org/wiki/Horde_IMP_%28email_client%29');
		$uri['iplanet']		 = t('http://en.wikipedia.org/wiki/IPlanet');
		$uri['kmail']		   = t('http://en.wikipedia.org/wiki/Kmail');
		$uri['lotus_notes']	 = t('http://en.wikipedia.org/wiki/Lotus_Notes');
		$uri['mozilla']		 = t('http://en.wikipedia.org/wiki/Mozilla');
		$uri['ms_exchange']	 = t('http://en.wikipedia.org/wiki/Microsoft_Exchange');
		$uri['mutt']			= t('http://en.wikipedia.org/wiki/Mutt_%28email_client%29');
		$uri['outlook']		 = t('http://en.wikipedia.org/wiki/Microsoft_Outlook');
		$uri['pine']			= t('http://en.wikipedia.org/wiki/Pine_%28email_client%29');
		$uri['squirrelmail']	= t('http://en.wikipedia.org/wiki/Squirrelmail');
		$uri['thunderbird']	 = t('http://en.wikipedia.org/wiki/Mozilla_Thunderbird');
	};
	return $uri[$code];
}; 
// }}}
// function get_os_uri($code) {{{
/**
 * Get an operating system URI from an internal code
 * @param string $code Internal OS code
 * @return string URI to a page describing the operating system
 * @see function extract_os_from_email_header
 */
function get_os_uri($code) {
	$uri = array();
	$uri['linux'] = t('http://en.wikipedia.org/wiki/Gnu/linux');
	$uri['windows'] = t('http://en.wikipedia.org/wiki/Microsoft_windows');
	return $uri[$code];
}; 
// }}}
// function t($str) {{{ 
/**
 * Translates a string, just like gettext's _() does,
 * using the global language parameter to select which
 * language to get the translation from.
 * @global string Function pointer to the specific language translation
 * @param string $str The string to translate
 * @return string The translated string
 */
function t($str) { 
	global $language_callback;
	if ($language_callback == 't_en') { return $str; };
	return $language_callback($str);
}; 
// }}}
// function tn($str1, $str2, $n) {{{
/**
 * Translates a plural string, just like gettext's ngettext() does,
 * using the global language parameter to select which
 * language to get the translation from.
 * @global string Function pointer to the specific language translation
 * @param string $str1 The string to translate on singular forms
 * @param string $str2 The string to translate on plural forms
 * @param integer $n The number to link to translation (2 and more are plural, all others singular)
 * @return string The translated string
 */
function tn($str1, $str2, $n) { 
	global $language_callback;
	if ($n > 1) {
		if ($language_callback == 't_en') { return $str2; };
		return $language_callback($str2);
	}
	else {
		if ($language_callback == 't_en') { return $str1; };
		return $language_callback($str1);
	};
}; 
// }}}
// function t_fr($str) {{{ 
/**
 * Translates a string, just like gettext's _() does, in french
 * @param string $str The string to translate
 * @return string The translated string
 */
function t_fr($str) {
	static $trans;
	if (count($trans) == 0) { 
		$trans = array();
		$trans['Untitled'] = 'Sans titre';
		$trans['Index'] = 'Index';
		$trans['signature'] = 'signature';
		$trans['[Index]'] = '[Index]';
		$trans['[Parent]'] = '[Parent]';
		$trans['[Prev]'] = '[Pr&#233;c]';
		$trans['[Next]'] = '[Suiv]';
		$trans['Attachments'] = 'Pi&#232;ces jointes';
		$trans['PGP public key'] = 'Clef publique PGP';
		$trans['PGP signature'] = 'Signature PGP';
		$trans['Untitled text attachment'] = 'Pi&#232;ce jointe sans nom';
		$trans['untitled'] = 'sans_titre';
		$trans['Permalink'] = 'Lien permanent';
		$trans['[%d thread]'] = '[%d fil de discussion]';
		$trans['[%d threads]'] = '[%d fils de discussion]';
		$trans['[%d message in thread]'] = '[%d message dans le fil]';
		$trans['[%d messages in thread]'] = '[%d messages dans le fil]';
	};
	return (((isset($trans[$str])) && ($trans[$str] != '')) ? $trans[$str] : $str);

}; 
// }}}
// function show_help_text() {{{
/**
 * Returns help text on standard out
 * @see function parse_options
 */
function show_help_text() {
	// this is plainly copy-pasted from the txt generated doc.
	echo <<<EOD
xhtmail transforms a collection of mail files into multiple XHTML webpages.

   Usage:

   php xhtmail.php options [file...]

		  Each file is a separate mail message, with full headers.
		  You must specify at least one file or use the -m option.

   Options:

   -a FILENAME, --after=FILENAME
		  What content (typically XHTML) should be put after the
		  xhtmail output?

   -b FILENAME, --before=FILENAME
		  What content (typically XHTML) should be put before the
		  xhtmail output?

		  Special tags in the file will be replaced:

		  + <^TITLE^> will be replaced by the mailthread title
		  + <^CHARSET^> will be replaced by the document's
			charset
		  + ...

		  No <hN> tags should be found in this file because they
		  are already used by xhtmail for message titles.

   -c FILENAME, --contents-after=FILENAME
		  What content (typically XHTML) should be put after the
		  thread contents in thread pages?

   -e EXTENSION, --extension=EXTENSION
		  What extension should we give to output files? Default
		  is html

   -f NUMBER, --feed=NUMBER
		  Number of Atom/RSS entries in the main archive output.
		  Defaults to 0 (no feed)

		  See -u option

   -h, --help
		  Displays this help text and exits

   -i FILENAME, --index_after=FILENAME
		  What content (typically XHTML) should be put after the
		  index in index pages?

   -l LANGUAGE, --lang=LANGUAGE
		  What language should the generated pages use? Default
		  is en

		  Available: en, fr

   -m STRING, --mail_filelist=STRING
		  File containing mail files to process, one per line

   -n STRING, --name_of_list=STRING
		  Name of the mailing list

   -o DIRECTORY, --output_directory=DIRECTORY
		  Where should the output files be placed?

   -p URI[,URI], --picture_uri=URI[,URI]
		  Image URI and icon URI respectively for the Atom/RSS
		  feeds. Useless if not used with -f; the icon should be
		  small, like 16x16, if provided

   -t STRING, --title_of_list=STRING
		  Long name of the archive (Like "Archives of the foo
		  mailing list")

   -u URI,URI, --uri_base=URI,URI
		  Base URI for webpages and RSS respectively, separated
		  by a comma, like "-u
		  http://ex.com/path1,http://ex.com/path2"; no slash is
		  wanted at the end. This parameter is silently discarded
		  if not using the -f flag (feeds) but mandatory if so.
		  These base URIs are used only for cross-linking between
		  web pages and feeds, not for internal links. The feeds
		  base should be before the feeds subdirectory (if your
		  feeds are at http://example.com/feeds/example.xml, use
		  http://example.com)

   -v, --version
		  Output version information and exit

   Caution Danger Will Robinson


   Option parsing is primitive at best. Should results be weird,
   command line inspection is strongly suggested.

EOD;
}; 
// }}}
// function parse_options() {{{
/**
 * Reads the commandline and sets internal variables according to it
 * Hash contains:
 * <ul>
 * <li><var>['after']			</var> Filename of the template to append to the thread pages</li>
 * <li><var>['before']		   </var> Filename of the template to prepend to the index and thread pages</li>
 * <li><var>['contents_after']   </var> Filename of the template to append to the thread pages, before ['after']</li>
 * <li><var>['extension']		</var> File extensions used on generated files</li>
 * <li><var>['feed']			 </var> Number of entries in each feed to generate</li>
 * <li><var>['files']			 </var> List of mail files to read</li>
 * <li><var>['gravatar_fallback']			 </var> URL to avatar fallback (48x48!)</li>
 * <li><var>['icon_uri']		 </var> URI of the mailing list icon</li>
 * <li><var>['index_after']	  </var> Filename of the template to append to the index pages, before ['after']</li>
 * <li><var>['lang']			 </var> 2-letter language interface to use</li>
 * <li><var>['name_of_list']	 </var> Mailing list name for titles and such</li>
 * <li><var>['output_directory'] </var> Output directory for generated files</li>
 * <li><var>['picture_uri']	  </var> URI of the mailing list logo</li>
 * <li><var>['title_of_list']	</var> Long title of the list</li>
 * <li><var>['uri_base_web']	 </var> Absolute URI to the mailing list directory</li>
 * <li><var>['uri_base_feeds']   </var> Absolute URI to the feeds directory</li>
 * </ul>
 * @return array The options array
 */
function parse_options() {
	$options = bgetop(array("-a|--after:", "-b|--before:", "-c|--contents-after:", "-e|--extension:", "-f|--feed:", "-g|--gravatar_fallback:", "-h|--help", "-i|--index_after:", "-l|--lang:", "-m|--mail_filelist:", "-n|--name_of_list:", "-o|--output_directory:", "-p|--picture_uri:", "-t|--title_of_list:", "-u|uri_base:", "--version"));
	$parsed_options = array();
	// parse options
	if ((isset($options['version'])) && ($options['version'])) {
		echo "xhtmail version " . XHTMAIL_VERSION_NUMBER . ' [' . XHTMAIL_VERSION_CODENAME . "]\n";
		echo wordwrap(XHTMAIL_COPYRIGHT);
		exit();
	};
	if ($options['a'] != '') {
		$parsed_options['after'] = $options['a'];
	}
	elseif ($options['after'] != '') {
		$parsed_options['after'] = $options['after'];
	};
	if ($options['b'] != '') {
		$parsed_options['before'] = $options['b'];
	}
	elseif ($options['before'] != '') {
		$parsed_options['before'] = $options['before'];
	};
	if ($options['c'] != '') {
		$parsed_options['contents-after'] = $options['c'];
	}
	elseif ($options['contents-after'] != '') {
		$parsed_options['contents-after'] = $options['contents-after'];
	};
	if ((!$options['e']) && (!$options['extension'])) {
		$parsed_options['extension'] = 'html';
	}
	else {
		if ($options['e'] != '') { 
			$parsed_options['extension'] = $options['e']; 
		}
		else {
			$parsed_options['extension'] = $options['extension']; 
		};
	};
	if ((!$options['f']) && (!$options['feed'])) {
		$parsed_options['feed'] = 0;
	}
	else {
		if ($options['f'] != '') { 
			$parsed_options['feed'] = (int)$options['f']; 
		}
		else {
			$parsed_options['feed'] = (int)$options['feed']; 
		};
	};
	if ((!$options['g']) && (!$options['gravatar_fallback'])) {
		$parsed_options['gravatar_fallback'] = '';
	}
	else {
		if ($options['g'] != '') { 
			$parsed_options['gravatar_fallback'] = trim($options['g']);
		}
		else {
			$parsed_options['gravatar_fallback'] = trim($options['gravatar_fallback']);
		};
	};
	if ((isset($options['h']) && $options['h']) || (isset($options['help']) && $options['help'])) {
		show_help_text();
		exit();
	};
	if ($options['i'] != '') {
		$parsed_options['index_after'] = $options['i'];
	}
	elseif ($options['index_after'] != '') {
		$parsed_options['index_after'] = $options['index_after'];
	};
	if ((!$options['l']) && (!$options['lang'])) {
		$parsed_options['lang'] = 'en';
	}
	else {
		if ($options['l'] != '') { 
			$parsed_options['lang'] = $options['l']; 
		}
		else {
			$parsed_options['lang'] = $options['lang']; 
		};
	};
	if ($options['m'] != '') {
		$parsed_options['mail_filelist'] = $options['m'];
	}
	elseif ($options['mail_filelist'] != '') {
		$parsed_options['mail_filelist'] = $options['mail_filelist'];
	};
	if ($parsed_options['mail_filelist'] != '') {
		if (!is_readable($parsed_options['mail_filelist'])) {
			die("Filelist file is not readable!\n");
		}
		else {
			$parsed_options['files'] = file($parsed_options['mail_filelist'], FILE_IGNORE_NEW_LINES|FILE_SKIP_EMPTY_LINES);
		};
	};
	if ((!$options['n']) && (!$options['name_of_list'])) {
		die("You must specify a mailing list name!\n");
	}
	else {
		if ($options['n'] != '') { 
			$parsed_options['name_of_list'] = $options['n']; 
		}
		else {
			$parsed_options['name_of_list'] = $options['name_of_list']; 
		};
	};
	if ((!$options['o']) && (!$options['output_directory'])) {
		die("You must specify an output directory!\n");
	}
	else {
		if ($options['o'] != '') { 
			$parsed_options['output_directory'] = $options['o']; 
		}
		else {
			$parsed_options['output_directory'] = $options['output_directory']; 
		};
	};
	if ((!$options['p']) && (!$options['picture_uri'])) {
		$parsed_options['picture_uri'] = '';
		$parsed_options['icon_uri'] = '';
	}
	else {
		if ($options['p'] != '') { 
			$preparsed_option = $options['p'];
		}
		else {
			$preparsed_option = $options['picture_uri'];
		};
		$uri_list = explode(",", $preparsed_option);
		if (count($uri_list) == 1) { 
			$parsed_options['picture_uri'] = $uri_list[0];
			$parsed_options['icon_uri'] = '';
		}
		else {
			$parsed_options['picture_uri'] = $uri_list[0];
			$parsed_options['icon_uri'] = $uri_list[1];
		};
	};
	if ((!$options['t']) && (!$options['title_of_list'])) {
		die("You must specify a mailing list title!\n");
	}
	else {
		if ($options['n'] != '') { 
			$parsed_options['title_of_list'] = $options['t']; 
		}
		else {
			$parsed_options['title_of_list'] = $options['title_of_list']; 
		};
	};
	if ((!$options['u']) && (!$options['uri_base']) && ($parsed_options['feed'] > 0)) {
		die("You must specify -u (base uri) if using -f (RSS/Atom feeds).\n");
	}
	else {
		if ($options['u'] != '') { 
			$preparsed_option = $options['u']; 
		}
		else {
			$preparsed_option = $options['uri_base']; 
		};
		$uri_list = explode(",", $preparsed_option);
		if ((count($uri_list) != 2) && ($parsed_options['feed'] > 0)) { die("You must specify -u (base uri) if using -f (RSS/Atom feeds).  You need 2 uri...\n"); };
		$parsed_options['uri_base_web'] = $uri_list[0];
		$parsed_options['uri_base_feeds'] = $uri_list[1];
	};
	if ((isset($options['unknown'])) && (count($options['unknown']) != 0)) {
		if (!$parsed_options['files']) {
			$parsed_options['files'] = $options['unknown'];
		}
		else {
			$parsed_options['files'] = array_merge($parsed_options['files'], $options['unknown']);
		};
	};
	if (!$parsed_options['files']) {
		die("You must provide at least one filename or use the -m option!\n");
	};
	return $parsed_options;
}; 
// }}}
// function extract_mua_from_email_header($str) {{{
/**
 * Extract a mail user agent from the mail header
 * Output hash contains
 * <ul>
 * <li><var>[0]</var>Textual name</li>
 * <li><var>[1]</var>Internal code name</li>
 * </ul>
 * @param string $headers A message header hash
 * @return array Hash as described above
 */
function extract_mua_from_email_header($headers) {
	$mua = '';
	if ((isset($headers['x-mailer'])) && ($headers['x-mailer'] != '')) { 
		$mua = $headers['x-mailer'];
	}
	elseif ((isset($headers['user-agent'])) && ($headers['user-agent'] != '')) {
		$mua = $headers['user-agent'];
	}
	elseif ((isset($headers['domainkey-signature'])) && ($headers['domainkey-signature'] != '')) {
		$mua = $headers['domainkey-signature'];
	}
	else {
		// last recourse for crap like Pine
		$mua = $headers['message-id'];
	};
	if (stristr($mua, 'alpine')) { return array('Alpine', 'alpine'); };
	if (stristr($mua, 'eudora')) { return array('Eudora', 'eudora'); };
	if (stristr($mua, 'evolution')) { return array('Evolution', 'evolution'); };
	if (stristr($mua, 'gmail')) { return array('Gmail', 'google'); };
	if (stristr($mua, 'kmail')) { return array('KMail', 'kmail'); };
	if (stristr($mua, 'internet mail service')) { return array('Microsoft Exchange', 'ms_exchange'); };
	if (stristr($mua, 'internet messaging program')) { return array('IMP', 'imp'); };
	if (stristr($mua, 'iplanet')) { return array('iPlanet Messenger', 'iplanet'); };
	if (stristr($mua, 'lotus notes')) { return array('Lotus Notes', 'lotus_notes'); };
	if (stristr($mua, 'mutt')) { return array('Mutt', 'mutt'); };
	if (stristr($mua, 'outlook')) { return array('Microsoft Outlook', 'outlook'); };
	if (stristr($mua, 'pine')) { return array('Pine', 'pine'); };
	if (stristr($mua, 'thunderbird')) { return array('Mozilla Thunderbird', 'thunderbird'); };
	if (stristr($mua, 'squirrelmail')) { return array('SquirrelMail', 'squirrelmail'); };
	# after thunderbird
	if (stristr($mua, 'mozilla')) { return array('Mozilla', 'mozilla'); };
	return array('', '');
}; 
// }}}
// function extract_os_from_email_header($str) {{{
/**
 * Extract an operating system from a mail header line
 * Output hash contains
 * <ul>
 * <li><var>[0]</var>Textual name</li>
 * <li><var>[1]</var>Internal code name</li>
 * </ul>
 * @param string $headers A message header hash
 * @return array Hash as described above
 */
function extract_os_from_email_header($headers) {
	$os = '';
	if ((isset($headers['x-operating-system'])) && ($headers['x-operating-system'] != '')) { 
		$os = $headers['x-operating-system'];
	}
	elseif ((isset($headers['user-agent'])) && ($headers['user-agent'] != '')) {
		$os = $headers['user-agent'];
	}
	elseif ((isset($headers['x-mailer'])) && ($headers['x-mailer'] != '')) {
		$os = $headers['x-mailer'];
	}
	elseif (stristr($headers['message-id'], "pine.lnx")) {
		$os = "linux";
	};
	if (stristr($os, 'linux')) { return array('GNU/Linux', 'linux'); };
	if (stristr($os, 'win32')) { return array('Windows', 'windows'); };
	if (stristr($os, 'windows')) { return array('Windows', 'windows'); };
}; 
// }}}
// function extract_tld_from_email_header($str) {{{
/**
 * Extract the country name from a mail header line
 * Output hash contains
 * <ul>
 * <li><var>[0]</var>Textual name</li>
 * <li><var>[1]</var>Top-Level Domain name</li>
 * </ul>
 * @param string $str The header line
 * @return array Hash as described above
 */
function extract_tld_from_email_header($str) {
	$regs = array();
	$regex = '[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]{2,})+';
	eregi($regex, $str, $regs);
	$country_code = substr($regs[2], 1);
	return array(get_country_name_from_tld($country_code), $country_code);
}; 
// }}}
// function extract_xface_from_email_header($data, $options) {{{
/**
 * Extract an X-Face icon from a mail header line to a PNG image
 * @param string $data The header line
 * @param array $options Global options
 * @return string Path and filename to the X-Face PNG
 * @see function parse_options
 */
function extract_xface_from_email_header($data, $options) {
	static $face_hash;
	$key = md5($data);
	if ((!isset($face_hash[$key])) || ($face_hash[$key] == '')) {
		mkdir_if_needed($options['output_directory'] . '/attachments');
		exec("echo " . escapeshellarg($data) . " | { echo '/* Width=48, Height=48 */'; " . XHTMAIL_UNCOMPFACE . "; } | " . XHTMAIL_ICONTOPBM . " | " . XHTMAIL_CONVERT . " - " . $options['output_directory'] . "/attachments/xface-$key.png");
		$face_hash[$key] = 1;
	};
	$path = $options['uri_base_web'] . "/attachments/xface-$key.png";
	return $path;
};
// }}}
// function extract_favicon_from_email_header($data, $options) {{{
/**
 * Extract a Favicon from a mail X-URL header line to a PNG image
 * @param string $data The header line
 * @param array $options Global options
 * @return string Path and filename to the Favicon PNG
 * @see function parse_options
 */
function extract_favicon_from_email_header($data, $options) {
	#	static $favicon;
	#	$url = parse_url($data);
	#	$domain = $url['host'];
	#	$key = md5($domain);
	#	if ($favicon_hash[$key] != '') { return "../../attachments/favicon-$key.png"; };
	#	mkdir_if_needed($options['output_directory'] . '/attachments');
	#
	#	exec("echo " . escapeshellarg($data) . " | { echo '/* Width=48, Height=48 */'; " . XHTMAIL_UNCOMPFACE . "; } | " . XHTMAIL_ICONTOPBM . " | " . XHTMAIL_CONVERT . " - " . $options['output_directory'] . "/attachments/xface-$key.png");
	#	$face_hash[$key] = 1;
	#	return "../../attachments/xface-$key.png";
}; 
// }}}
// function extract_author_name_from_email_header($str) {{{
/**
 * Extract the author name from a mail header line
 * Will return the left-part of "@" if only an e-mail address is available
 * @param string $str The header line
 * @return string The author name
 */
function extract_author_name_from_email_header($str) { 
	// first, check if it's a <asdf>-only format
	if (preg_match("/^<.*>$/", $str)) {
		return substr($str, 1, strpos($str, '@') - 1);
	};
	// two times for formats like "abe <a@b.c> by way of bob <d@e.f>
	$str = preg_replace("/^(.*)<.*>.*$/", '\1', $str);
	$str = preg_replace("/^(.*)<.*>.*$/", '\1', $str);
	$str = str_replace('"', '', $str);
	// make sure there's no comma in e-mail, they are used
	// to separate entries in contributor list
	$str = str_replace(',', '', $str);
	$str = trim($str);
	// if there's an arobas left, remove it for everybody's sake
	if (($pos = strpos($str, "@")) !== FALSE) {
		$str = substr($str, 0, $pos);
	};
	return $str;
}; 
// }}}
// function extract_email_from_from_header($str) {{{
/**
 * Extract the email from a mail header line
 * @param string $str The from header line
 * @return string The e-mail address
 */
function extract_email_from_from_header($str) {
	$matches = array();
	$mailregex = preg_match('/^.*\b([A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,4})\b.*$/i', $str, $matches);
	return strtolower($matches[1]);
};
// }}}
// function mail_encoding_to_utf8($str, $charset, $qp = FALSE) {{{
/**
 * Transform mail-encoded contents to UTF-8
 * @param string $str The contents to recode
 * @param string $charset The original charset
 * @param boolean $qp Is the original charset specified in quoted-printable format?
 * @return string UTF-8 encoded string
 */
function mail_encoding_to_utf8($str, $charset, $qp = FALSE) {
	$prefix = '';
	$postfix = '';
	if ($qp == TRUE) { 
		$prefix = '=?';
		$postfix = '?';
	};
	if (stristr($charset, $prefix . "utf-8" . $postfix)) {
		// nothing to do
	}
	elseif (stristr($charset, $prefix . "iso-8859-1" . $postfix)) {
		$str = utf8_encode($str);
	}
	elseif (stristr($charset, $prefix . "iso-8859-6" . $postfix)) {
		$str = iconv("iso-8859-6", "utf-8", $str);
	}
	elseif (stristr($charset, $prefix . "windows-1252" . $postfix)) {
		$str = iconv("windows-1252", "utf-8", $str);
	}
	elseif (stristr($charset, $prefix . "windows-1256" . $postfix)) {
		$str = iconv("windows-1256", "utf-8", $str);
	}
	elseif (stristr($charset, $prefix . "windows-1258" . $postfix)) {
		$str = iconv("windows-1258", "utf-8", $str);
	}
	elseif (($qp && preg_match('/' . str_replace('?', '\?', $prefix) . ".*" . str_replace('?', '\?', $postfix) . '/', $str))) {
		die('unknown encoding: ' . $charset);
	};
	$str = replace_win_illegal_chars($str);
	return $str;
}; 
// }}}
// function replace_win_illegal_chars($str) {{{
/**
 * Remove windows characters in the 128-159 range.  They seem to crawl everywhere.
 * @param string $str The contents to recode.  It's supposed to be UTF-8, after conversion with iconv
 * @return string UTF-8 encoded string
 */

function replace_win_illegal_chars($str) {
	static $win_from = '';
	static $win_to = '';
	if ($win_from == '') {
		$win_from = array();
		$win_to = array();
		$win_from[] = chr(194) . chr(133);		$win_to[] = '...';
		$win_from[] = chr(194) . chr(146);		$win_to[] = "'";
		$win_from[] = chr(194) . chr(147);		$win_to[] = chr(194) . chr(171);
		$win_from[] = chr(194) . chr(148);		$win_to[] = chr(194) . chr(187);
		$win_from[] = chr(194) . chr(150);		$win_to[] = '-';
		$win_from[] = chr(197) . chr(147);		$win_to[] = 'oe';
		$win_from[] = chr(226) . chr(128) . chr(153);	$win_to[] = "'";
		$win_from[] = chr(226) . chr(128) . chr(166);	$win_to[] = "...";
		$win_from[] = chr(226) . chr(128) . chr(147);	$win_to[] = "-";
	};	
	return str_replace($win_from, $win_to, $str);
};
// }}}
// function is_utf8($str) {{{
/**
 * Checks a string for utf8ness
 * @param string $str The contents to check
 * @return boolean True is contents is UTF-8, false otherwise
 */
function is_utf8($str) {
	return (preg_match('/^([\x00-\x7f]|[\xc2-\xdf][\x80-\xbf]|\xe0[\xa0-\xbf][\x80-\xbf]|[\xe1-\xec][\x80-\xbf]{2}|\xed[\x80-\x9f][\x80-\xbf]|[\xee-\xef][\x80-\xbf]{2}|f0[\x90-\xbf][\x80-\xbf]{2}|[\xf1-\xf3][\x80-\xbf]{3}|\xf4[\x80-\x8f][\x80-\xbf]{2})*$/', $str) === 1);
}; 
// }}}
// function myurlencode($url) {{{
/**
 * Encodes a URL without encoding metachars like {'/', '?', '&'} 
 * @param string $url The URL to encode
 * @return string The encoded URL
 */
function myurlencode($url) {
	// urlencode only meaningful chars (like > 127) so we don't touch ?/ and other sensible thinggies
	$outurl = '';
	for ($i = 0; $i < strlen($url); $i++) {
		if ((ord($url[$i]) > 127) || (ord($url[$i]) == 32) || (ord($url[$i]) == 91) || (ord($url[$i]) == 93)) {
			$outurl .= urlencode($url[$i]);
		}
		else {
			$outurl .= $url[$i];
		};
	};
	return $outurl;
}; 
// }}}
// function bgetop($opAr = array(), $unknown = true) {{{
/**
 * Better getopt
 * Parses commandline options
 * @author Pete Higgins
 * @link http://higginsforpresident.net/projects/source/getopt-0.1.phps
 * @param array $opAr Array with tokens to match
 * @param boolean $unknown Are we expecting non-flagged values like filenames?
 * @return mixed Array of parameters on success, FALSE otherwise
 */
function bgetop($opAr = array(), $unknown = true) {
	$argv = $_SERVER['argv'];
	$argc = $_SERVER['argc'];
	$argPos = 1; // zero is program running

	// foreach arg
	while ($argPos < $argc) {
		$arg = $argv[$argPos];
		if ($arg{0} == "-") {
		   if ($arg{1} == "-") {
			   $var = substr($arg, 2, strlen($arg));
		   } 
		   else { 
			   $var = $arg{1}; 
		   }
		   foreach ($opAr as $opk => $opv) {
			   if (!isset($return[$var])) {
				   if (strpos($opv,$arg) !== FALSE) {
					   // this is where the -f -foo fix needs to be,
					   // the partial string exists in this record,
					   // but we need to determine if it's accurate
					   // somehow (i'm thinking: eregi?)
					   if ($accurate=1) {
						   // we foudn the key
						   if (strpos($opv, ':') !== FALSE) {
							   // next value is the one to use,
							   // then skip it in the parser.
							   if (isset($argv[$argPos+1])) {
								   $return[$var] = $argv[++$argPos];
							   } 
							   else {
								   $return[$var] = FALSE;
							   };
						   } 
						   else {
							   // just set the toggle
							   $return[$var] = TRUE;
						   };
						   // don't check this opAr value again
						   unset($opAr[$opk]);
					   };
				   }; // if accurate
			   }; // !isset already
		   }; // foreach opAr
		} 
		else { // we weren't expecting a non-hyphened argument, possibly just a filename, or whatnot
			if ($unknown) { 
				$return['unknown'][] = $arg; 
			};
		};
		$argPos++;
	}; // while argPos < argc
	if (is_array($return)) {
		return $return;
	} 
	else { 
		return 0; 
	};
}; 
// }}}
// function myxmlentities($string) {{{
/**
 * XML-encodes a string (like htmlentities), encoding chars like amperstand and backslash too
 * @param string $string The original string
 * @return string The encoded string
 */
function myxmlentities($string) {
	static $transxml_from = '';
	static $transxml_to = '';
	if ($transxml_from == '') {
		$transxml = get_html_translation_table(HTML_ENTITIES, ENT_QUOTES);
		$transxml_from = array();
		$transxml_to = array();
		$transxml_from[] = "&";	   $transxml_to[] = "&#38;";
		$transxml_from[] = '\\';	  $transxml_to[] = "&#92;";
		$transxml_from[] = chr(146);  $transxml_to[] = "&#39;";
		$transxml_from[] = chr(133);  $transxml_to[] = "";
		foreach ($transxml as $key => $value) { 
			// amperstand should be at the very start; if not, strange things WILL happen.
			if ($key != "&") {
				$transxml_from[] = $key;
				$transxml_to[] = '&#'.ord($key).';';
			};
		};	
	};
	// using str_replace instead of strtr seems MUCH faster.
	return str_replace($transxml_from, $transxml_to, $string);
	//return strtr($string, $transxml);
}; 
// }}}
// function htmlwrap($str, $width = 60, $break = "\n", $nobreak = "", $nobr = "pre", $utf = FALSE) {{{
/**
 * Wordwraps a string, with consideration for XML tags
 * @author Brian Huisman AKA GreyWyvern
 * @link http://www.greywyvern.com/php.php
 * @param string $str The string to wordwrap
 * @param integer $width The width to wrap to
 * @param string $break The character(s) to break lines with
 * @param string $nobreak List of space-delimited tags that won't be broken
 * @param string $nobr List of space-delimited block-tags that won't be broken
 * @param boolean $utf Do we use multi-byte logic to calculate lengths
 * @return string The wrapped string
 */
function htmlwrap($str, $width = 60, $break = "\n", $nobreak = "", $nobr = "pre", $utf = FALSE) {
  if (XHTMAIL_DEBUG) { echo "\tEntering htmlwrap\n"; };
  // Split HTML content into an array delimited by < and >
  // The flags save the delimeters and remove empty variables
  $content = preg_split("/([<>])/", $str, -1, PREG_SPLIT_DELIM_CAPTURE | PREG_SPLIT_NO_EMPTY);

  // Transform protected element lists into arrays
  $nobreak = explode(" ", $nobreak);
  $nobr = explode(" ", $nobr);

  // Variable setup
  $intag = false;
  $innbk = array();
  $innbr = array();
  $drain = "";
  $utf = ($utf) ? "u" : "";

  // List of characters it is "safe" to insert line-breaks at
  // Do not add ampersand (&) as it will mess up HTML Entities
  // It is not necessary to add < and >
  $lbrks = "/?!%)-}]\\\"':;";

  // We use \r for adding <br /> in the right spots so just switch to \n
  if ($break == "\r") $break = "\n";

  while (list(, $value) = each($content)) {
	switch ($value) {

	  // If a < is encountered, set the "in-tag" flag
	  case "<": $intag = true; break;

	  // If a > is encountered, remove the flag
	  case ">": $intag = false; break;

	  default:

		// If we are currently within a tag...
		if ($intag) {

		  // If the first character is not a / then this is an opening tag
		  if ($value{0} != "/") {

			// Collect the tag name   
			preg_match("/^(.*?)(\s|$)/$utf", $value, $t);

			// If this is a protected element, activate the associated protection flag
			if ((!count($innbk) && in_array($t[1], $nobreak)) || in_array($t[1], $innbk)) $innbk[] = $t[1];
			if ((!count($innbr) && in_array($t[1], $nobr)) || in_array($t[1], $innbr)) $innbr[] = $t[1];

		  // Otherwise this is a closing tag
		  } else {

			// If this is a closing tag for a protected element, unset the flag
			if (in_array(substr($value, 1), $innbk)) unset($innbk[count($innbk)]);
			if (in_array(substr($value, 1), $innbr)) unset($innbr[count($innbr)]);
		  }

		// Else if we're outside any tags...
		} else if ($value) {

		  // If unprotected, remove all existing \r, replace all existing \n with \r
		  if (!count($innbr)) $value = str_replace("\n", "\r", str_replace("\r", "", $value));

		  // If unprotected, enter the line-break loop
		  if (!count($innbk)) {
			do {
			  $store = $value;

			  // Find the first stretch of characters over the $width limit
			  if (preg_match("/^(.*?\s|^)(([^\s&]|&(\w{2,5}|#\d{2,4});){".$width."})(?!(".preg_quote($break, "/")."|\s))(.*)$/s$utf", $value, $match)) {

				// Determine the last "safe line-break" character within this match
				for ($x = 0, $ledge = 0; $x < strlen($lbrks); $x++) $ledge = max($ledge, strrpos($match[2], $lbrks{$x}));
				if (!$ledge) $ledge = strlen($match[2]) - 1;

				// Insert the modified string
				$value = $match[1].substr($match[2], 0, $ledge + 1).$break.substr($match[2], $ledge + 1).$match[6];
			  }

			// Loop while overlimit strings are still being found
			} while ($store != $value);
		  }

		  // If unprotected, replace all \r with <br />\n to finish
		  if (!count($innbr)) $value = str_replace("\r", "\n", $value);
		}
	}

	// Send the modified segment down the drain
	$drain .= $value;
  }

  // Return contents of the drain
  return $drain;
} 
// }}}
// function get_description_from_body($body) {{{
/**
 * Takes a message body and filters it for feed inclusion
 * @param string $body The message body
 * @param integer $number_of_lines Number of lines to return
 * @return string The filtered body
 */
function get_description_from_body($body, $number_of_lines) {
	$lines_in = explode("\n", $body);
	$nol = count($lines_in);
	$lines_out = array();
	for ($i = 0; $i < $nol; $i++) {
		$l = $lines_in[$i];
		if (ereg("^<div class=\"sig\">", $l)) {
			break;
		};
		if (ereg("^<blockquote>", $l)) {
			while (!ereg("</blockquote>$", $l)) {
				$l = $lines_in[++$i];
			};
			continue;
		};
		if (ereg("^&#62;", $l)) {
			while (ereg("^&#62;", $l)) {
				$l = $lines_in[++$i];
			};
			$i--;
			continue;
		};
		// empty lines become breaks
		if (trim($l) == '') {
			$lines_out[] = "<br /><br />";
		}
		else {
			$lines_out[] = trim($l);
		};
	};

	// get the first three lines
	if (count($lines_out) < $number_of_lines) { $number_of_lines = count($lines_out); };
	$out = array();
	for ($i = 0; $i < $number_of_lines; $i++) {
		// in theory, if there is an opening <a>, the closing </a> is in the same scope
		// so we can add the lines upto the closing tab
		$preout = '';
		if (!ereg("<a.*</a>", $lines_out[$i])) {
			if (ereg("<a", $lines_out[$i])) {
				$preout = $lines_out[$i];
				for ($j = 1; $j < ($nol - $i); $j++) {
					if (ereg("</a>", $lines_out[$i + $j])) {
						$i += $j;
						break;
					};
					$preout .= $lines_out[$i + $j];
				};

			};
		};
		$out[] = $preout . $lines_out[$i];
	};
	// add ... to last line if not real last
	if ($i < count($lines_out)) {
		$out[count($out) - 1] .= " [...]";
	};
	return preg_replace('/<br \/><br \/>$/', '', implode(" \n", str_replace("<br /><br /> [...]", "[...]", $out)));
}; 
// }}}
// function get_iso8601_date($int_date) {{{
/** 
 * Transforms a unix timestamp date into a ISO8601 formatted date
 * @param int $int_date The unix timestamp
 * @return string The ISO8601 formatted date
 */
function get_iso8601_date($int_date) { 
	//$int_date: current date in UNIX timestamp
	$date_mod = date('Y-m-d\TH:i:s', $int_date);
	$pre_timezone = date('O', $int_date);
	$time_zone = substr($pre_timezone, 0, 3).":".substr($pre_timezone, 3, 2);
	$date_mod .= $time_zone;
	return $date_mod;
}; 
// }}}
// function format_xhtml_from_mail($string) {{{
/**
 * Transform a XHTML mailpart to text
 * @param string $htmlbody The XHTML part
 * @return string The text extracted
 */
function format_xhtml_from_mail($htmlbody) {
	$out = '';
	$descriptorspec = array(
		0 => array("pipe", "r"),  // stdin
		1 => array("pipe", "w"),  // stdout
		2 => array("pipe", "w")  // stderr ?? instead of a file
	);
	$process = proc_open(XHTMAIL_XHTML_TO_TEXT, $descriptorspec, $pipes);
	if (is_resource($process)) {
		fwrite($pipes[0], $htmlbody);
		fclose($pipes[0]);
		while ($s = fgets($pipes[1], 1024)) {
			$out .= $s;
		};
		fclose($pipes[1]);
		fclose($pipes[2]);
	};
	return $out;
};
// }}}
function mail_wordwrap($msg, $width) { /* {{{ */
	$out = array();
	$lines = explode("\n", $msg);
	// Add an empty line, if last line is not empty
	// the end of the text will be repeated for nothing.
	$lines[count($lines)] = '';
	for ($i = count($lines) - 1; $i >= 0; $i--) {
		$lines[$i] = rtrim($lines[$i]);
	};
	for ($i = 0; $i < count($lines); $i++) {
		/* Clear buffer. */
		$buf = array();
		/* Empty line, just append to output */
		if ($lines[$i] == '') {
			$out[] = '';
			continue;
		};
		/* Line is not empty, find data up to the next empty line */
		$buf[] = $lines[$i];
		for ($j = $i + 1; $j < count($lines); $j++) {
			if ($lines[$j] != '') {
				$buf[] = $lines[$j];
			}
			else {
				$i = $j;
				break;
			};
		};
		$out = array_merge($out, mail_wordwrap_paragraph($buf, $width));
		$out[] = '';
	};
	return implode("\n", $out);
}; /* }}} */
function mail_wordwrap_paragraph($lines, $width) { /* {{{ */
	/* First, check if everything fits.  It does? return! */
	$okay = TRUE;
	if (!count($lines)) { return array(); };
	foreach ($lines as $l) {
		#echo strlen($l) . "\n";
		if (strlen($l) >= $width) {
			$okay = FALSE;
			break;
		};
	};
	if ($okay) {
		#echo "PARAGRAPH WAS OKAY\n";
		return $lines;
	};

	/* This is mail, so there might be quoting characters at the left. */
	/* We define quote marks as anything before a space that ends with '>'
	   and is repeated on all the lines, max 6 chars */

	$outlines = array();
	$matches = array();
	preg_match('/^(.?.?.?.?.?>)/', $lines[0], $matches);
	/* If quote character matches, find all lines with that quote character, then wrap */
	if (isset($matches[1])) {
		$buf = array();
		$okay = TRUE;
		while ($okay == TRUE) {
			if (substr($lines[0], 0, strlen($matches[1])) != $matches[1]) {
				$okay = FALSE;
			}
			else {
				$buf[] = array_shift($lines);
			};
			if (!(count($lines))) { $okay = FALSE; };
		};
		#echo "found match : " . $matches[1] . "\n";
		$temp_lines = array();
		foreach ($buf as $l) {
			$temp_lines[] = substr($l, strlen($matches[1]));
		};
		$wrapped = phpmailer_wraptext(implode("\n", $temp_lines), $width - strlen($matches[1]));
		$wrapped = explode("\n", trim($wrapped));
		foreach ($wrapped as $w) {
			$outlines[] = $matches[1] . ' ' . ltrim($w);
		};
	}
	else {
		/* quote character doesn't match.  find all lines without a quote character, then wrap */
		$okay = TRUE;
		$buf = array();
		while ($okay == TRUE) {
			$matches = array();
			preg_match('/^(.?.?.?.?.?>)/', $lines[0], $matches);
			if (isset($matches[1])) {
				$okay = FALSE;
			}
			else {
				$buf[] = array_shift($lines);
			};
			if (!(count($lines))) { $okay = FALSE; };
		};
		$wrapped = phpmailer_wraptext(implode("\n", $buf), $width);
		$wrapped = explode("\n", trim($wrapped));
		foreach ($wrapped as $w) {
			$outlines[] = ltrim($w);
		};
	};
	
	/* recurse with remaining lines */
	$outlines = array_merge($outlines, mail_wordwrap_paragraph($lines, $width));

	return $outlines;
}; /* }}} */
function phpmailer_wraptext($message, $length, $qp_mode = false) { /* {{{ */
	/* Function imported from phpmailer http://phpmailer.sourceforge.net/ under LGPL */
	$LE = "\n";
	$soft_break = ($qp_mode) ? sprintf(" =%s", $LE) : $LE;

	$message = phpmailer_fixeol($message);
	if (substr($message, -1) == $LE) {
		$message = substr($message, 0, -1);
	};

	$line = explode($LE, $message);
	$message = "";
	for ($i=0 ;$i < count($line); $i++) {
		$line_part = explode(" ", $line[$i]);
		$buf = "";
		for ($e = 0; $e < count($line_part); $e++) {
			$word = $line_part[$e];
			if ($qp_mode and (strlen($word) > $length)) {
				$space_left = $length - strlen($buf) - 1;
				if ($e != 0) {
					if ($space_left > 20) {
						$len = $space_left;
						if (substr($word, $len - 1, 1) == "=") {
							$len--;
						}
						elseif (substr($word, $len - 2, 1) == "=") {
							$len -= 2;
						};
						$part = substr($word, 0, $len);
						$word = substr($word, $len);
						$buf .= " " . $part;
						$message .= $buf . sprintf("=%s", $LE);
					}
					else {
						$message .= $buf . $soft_break;
					};
					$buf = "";
				}
				while (strlen($word) > 0) {
					$len = $length;
					if (substr($word, $len - 1, 1) == "=") {
						$len--;
					}
					elseif (substr($word, $len - 2, 1) == "=") {
						$len -= 2;
					};
					$part = substr($word, 0, $len);
					$word = substr($word, $len);
					if (strlen($word) > 0) {
						$message .= $part . sprintf("=%s", $LE);
					}
					else {
						$buf = $part;
					};
				}
			}
			else {
				$buf_o = $buf;
				$buf .= ($e == 0) ? $word : (" " . $word);
				if (strlen($buf) > $length and $buf_o != "") {
					$message .= $buf_o . $soft_break;
					$buf = $word;
				};
			};
		};
		$message .= $buf . $LE;
	};
	return $message;
}; /* }}} */
function phpmailer_fixeol($str) { /* {{{ */
	/* Function imported from phpmailer http://phpmailer.sourceforge.net/ under LGPL */
	$str = str_replace("\r\n", "\n", $str);
	$str = str_replace("\r", "\n", $str);
	return $str;
}; /* }}} */
// }}}
?>
